/*
 * Copyright (c) 2008 NVIDIA Corporation.  All Rights Reserved.
 *
 * NVIDIA Corporation and its licensors retain all intellectual property and
 * proprietary rights in and to this software and related documentation.  Any
 * use, reproduction, disclosure or distribution of this software and related
 * documentation without an express license agreement from NVIDIA Corporation
 * is strictly prohibited.
 */

#define _WIN32_IE 0x400
#include <windows.h>
#include <commctrl.h>
#include <streams.h>
//#include "nvdshow_encode.h"
#include "nvencode_test.h"
#include "nvencode_test_defines.h"
//#include <uuids.h>


#define ACA		1
#define ACM		0

//please disable WITH_STILL_CAPTURE for 1.4 or earlier release image.
//2.0 beta 1 image support still encoder support
#define WITH_STILL_CAPTURE		0
#define USE_VIDEO_WINDOW		1
#define AUDIO_CAPTURE			0

#if ACA
#define CROP_BORDER 			1
#endif

// seems like we can capture at most 640x480, with 200kbps or more.
// TODO: test a little further.
#define DEFAULT_BITRATE		10000000
//#define DEFAULT_BITRATE			200000

#if ACA
#define CAMERA_WIDTH	800
#define CAMERA_HEIGHT	624
#endif

#if ACM
#define CAMERA_WIDTH	720
#define CAMERA_HEIGHT	240
#endif

DWORD g_CaptureFPS=30;
DWORD g_CaptureWidth=CAMERA_WIDTH;
DWORD g_CaptureHeight=CAMERA_HEIGHT;
BOOL  g_CaptureEnable=TRUE;
DWORD g_PreviewWidth=CAMERA_WIDTH;
DWORD g_PreviewHeight=CAMERA_HEIGHT;
BOOL  g_PreviewEnable=TRUE;

//please disable(false) WITH_STILL_CAPTURE for 1.4 or earlier release image.
//2.0 beta 1 image supports still encoder support, so still can be TRUE
//BOOL  g_StillEnable=FALSE;
BOOL  g_AudioEnable=FALSE;

BOOL  g_VideoWindow=TRUE;
DWORD g_Duration = 5; // Default record time = 5 sec
DWORD g_PreviewDuration = 5; // Run preview for 5 secs before starting capture
char g_OutFilename[256] = "out.3gp"; // Default output filename
char g_outFileStillName[256] = "temp.jpg";

//Global Variables
IGraphBuilder *m_pGraph = NULL;
IMediaEventEx *m_pEventEx = NULL;
extern IVideoWindow  *g_videoWindow;
IBaseFilter *m_pSourceFilter = NULL;
IBaseFilter *m_pAudSourceFilter = NULL;
IBaseFilter *m_pEncoderFilterStill = NULL;
IBaseFilter *m_pEncoderFilterVideo = NULL;
IBaseFilter *m_pAudEncoderFilter = NULL;
IBaseFilter *m_pRendererFilter = NULL;
IKsPropertySet *m_pPropSetFilter = NULL;
IMediaControl *m_pControl = NULL;
ICaptureGraphBuilder2 *m_pCaptureGraphBuilder = NULL;
IBaseFilter *m_pSinkFilterStill = NULL;
IBaseFilter *m_pSinkFilterVideo = NULL;
IBasicVideo *m_pVideoRendererBasicVideo = NULL;
//IBaseFilter *m_pVideoCapture = NULL;
IMediaEvent *m_pEvent = NULL;
IFileSinkFilter *m_pFileSinkFilter = NULL;

HRESULT	SetSourceFilterPinsMediaType(IBaseFilter *pSourceFilter, AM_MEDIA_TYPE *pPreviewMediaType, AM_MEDIA_TYPE *pCaptureMediaType, AM_MEDIA_TYPE *pStillMediaType);
HRESULT NvEncodeTest(HWND ghApp);
HWND	CreateVideoWindow();
HRESULT SetupVideoWindow();
void	DumpGraphEx(const IGraphBuilder *pGraph);
HRESULT  TriggerPin(GUID PinCategory );
void SelectSensor(BOOL bForceDialog);
void GetSensorMode(TCHAR* pszPinNameMode, DWORD* pWidth, DWORD* pHeight, DWORD* pFps, BOOL bForceDialog);

#define FAILEDHR(hr,message,label)     {if (FAILED(hr)){printf(message, hr);goto label;}}

//int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
//                    LPWSTR lpCmdLine, int nCmdShow)
int wmain(int argc, char* argv[])
{
    int n = 0;
    HWND ghApp = 0;

    while(n<argc)
    {
        if(strcmp(argv[n], "audio")==0)
        {
            n++;
            g_AudioEnable=TRUE;
        }
        else if(strcmp(argv[n], "help")==0)
        {
            n++;
            printf("CameraDemo.exe help information\r\n");
            printf("still\tStill support\r\n");
            printf("audio\taudio capturing support\r\n");
            printf("nogui\tConsole Application\r\n");
            printf("press enter key to exit");
            getchar();
            return 0;
        }
        else if(strcmp(argv[n], "nogui")==0)
        {
            n++;
            g_VideoWindow = FALSE;			
        }
        else
        {
            n++;
        }
    } 

    if(g_VideoWindow)
    {
        INITCOMMONCONTROLSEX icex;
        icex.dwSize = sizeof(INITCOMMONCONTROLSEX);
        icex.dwICC = ICC_BAR_CLASSES;
        if (!InitCommonControlsEx(&icex))
        {
            int err = GetLastError();
            printf("Error initializing CommonControls: %i.\n", err);
        }
        SelectSensor(GetAsyncKeyState(VK_SHIFT)&0x80000000);
        GetSensorMode(L"CaptureMode", &g_CaptureWidth, &g_CaptureHeight, &g_CaptureFPS, GetAsyncKeyState(VK_SHIFT)&0x80000000);
        g_PreviewWidth = g_CaptureWidth;
        g_PreviewHeight = g_CaptureHeight;
    }

    if(g_VideoWindow)
    {
        ghApp = CreateVideoWindow();
        //ShowWindow(ghApp, 5);
        if (ghApp == 0) { printf("Could not initialize preview window. Exiting (Press ENTER)\n"); getchar();return 0;}
    }

    HRESULT res = NvEncodeTest (ghApp);
    if (res != S_OK) getchar();
    return 0;
}

HRESULT NvEncodeTest(HWND ghApp)
{
    HRESULT hr = S_OK;

    if(g_VideoWindow)
    {
        //printf("Close preview window to stop the demo\n");
    }
    else
    {
        printf("Press Enter in this console window to stop the demo\n");
        printf("Enter 'c' - Video capture start, 'x' - Video capture end, 's' - Still capture,'e' - exit application\n");
    }

    //if (g_CaptureEnable) printf("\nOutputfile is %s\n", g_OutFilename);


    //if(!g_VideoWindow)
    //{
    //Create Instance of COM to be used for this Test.
    hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);
    FAILEDHR((hr), "CoInitializeEx failed.\n", CleanUp);
    //}

    // Create Instance of Filter Graph Manager to handle AV Recording Filter Graph.
    hr = CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER, IID_IGraphBuilder, (void **)&m_pGraph);
    FAILEDHR(hr, "IGraphBuilder's CoCreateInstance failed.\n", CleanUp);

    // Create instance of Capture Graph builder
    hr = CoCreateInstance(CLSID_CaptureGraphBuilder, NULL, CLSCTX_INPROC_SERVER, IID_ICaptureGraphBuilder2, (void**)&m_pCaptureGraphBuilder);
    FAILEDHR(hr, "CaptureGraphBuilder's CoCreateInstance failed.\n", CleanUp);

    // Set Filter graph manager in the Capture Graph builder
    m_pCaptureGraphBuilder->SetFiltergraph(m_pGraph);

    // dump graph building to a log file (not very helpfull)
    //pGraph->SetLogFile(CreateFile(TEXT("\\graph_builder.log"), GENERIC_WRITE, 0, NULL, CREATE_NEW, FILE_ATTRIBUTE_NORMAL, NULL));

    //// Here Sink Filter is 3GP Muxer which also writes to file.
    hr = AddSinkFilter(m_pGraph, &m_pSinkFilterStill, 1);
    FAILEDHR(hr, "Unable to add sink filter to graph.\n", CleanUp);

    hr = AddSinkFilter(m_pGraph, &m_pSinkFilterVideo, 0);
    FAILEDHR(hr, "Unable to add sink filter to graph.\n", CleanUp);

    hr = RenderVideoInGraph(m_pGraph, m_pCaptureGraphBuilder, m_pSinkFilterStill, m_pSinkFilterVideo);
    FAILEDHR(hr, "Render Video in the graph failed.\n", CleanUp);

    //#if	AUDIO_CAPTURE
    //   hr = RenderAudioInGraph(m_pGraph, m_pCaptureGraphBuilder, m_pSinkFilter);
    //   FAILEDHR(hr, "Render Audio in the graph failed.\n", CleanUp);
    //#endif

#if 0 //CROP_BORDER
    m_pGraph->QueryInterface(__uuidof(IBasicVideo),(void **)&m_pVideoRendererBasicVideo);
    LONG top;
    LONG width;
    LONG left;
    LONG height;

    m_pVideoRendererBasicVideo->GetSourcePosition(&left,&top,&width,&height);
    m_pVideoRendererBasicVideo->GetDestinationPosition(&left,&top,&width,&height);

    m_pVideoRendererBasicVideo->SetSourcePosition(70,40,702,576);
    m_pVideoRendererBasicVideo->SetDestinationPosition(20,20,702,576);

    m_pVideoRendererBasicVideo->GetSourcePosition(&left,&top,&width,&height);
    m_pVideoRendererBasicVideo->GetDestinationPosition(&left,&top,&width,&height);
#endif

    hr = RunGraph(ghApp, m_pGraph);
    FAILEDHR(hr, "Run Graph failed (%x).\n", CleanUp);

CleanUp:
    if (m_pGraph != NULL) m_pGraph->Release();
    if (m_pCaptureGraphBuilder != NULL) m_pCaptureGraphBuilder->Release();
    if (m_pSinkFilterStill != NULL) m_pSinkFilterStill->Release();
    if (m_pSinkFilterVideo != NULL) m_pSinkFilterVideo->Release();
    CoUninitialize();
    return hr;
}


HRESULT
RenderVideoInGraph(
                   IGraphBuilder *pGraph,
                   ICaptureGraphBuilder2 *pCaptureGraphBuilder,
                   IBaseFilter *pSinkFilterStill,
                   IBaseFilter *pSinkFilterVideo)
{

    AM_MEDIA_TYPE VideoMediaType = {0};
    VIDEOINFOHEADER VideoInfo = {0};
    AM_MEDIA_TYPE StillMediaType = {0};
    VIDEOINFOHEADER StillInfo = {0};
    AM_MEDIA_TYPE PreviewMediaType = {0};
    VIDEOINFOHEADER PreviewInfo = {0};
    HRESULT hr = E_FAIL;

    WORD VideoFrameRate = (WORD)g_CaptureFPS; // Default is 30 FPS
    WORD VideoWidth = (WORD)g_CaptureWidth; //720; // Default video resolution
    WORD VideoHeight = (WORD)g_CaptureHeight; ///Not deinterlaced yet //486;

    WORD StillWidth = (WORD)g_CaptureWidth; //720; // Default video resolution
    WORD StillHeight = (WORD)g_CaptureHeight; ///Not deinterlaced yet //486;

    WORD PreviewWidth = (WORD) g_PreviewWidth; //(640);//720; // Default preview resolution
    WORD PreviewHeight = (WORD) g_PreviewHeight;///Not deinterlaced yet //486;

    //NV_ASSERT(pGraph != NULL);
    //NV_ASSERT(pCaptureGraphBuilder != NULL);
    //NV_ASSERT(pSinkFilter != NULL);

    VideoInfo.AvgTimePerFrame = UNITS / VideoFrameRate;
    VideoInfo.bmiHeader.biWidth = VideoWidth;
    VideoInfo.bmiHeader.biHeight = VideoHeight;
    VideoInfo.bmiHeader.biBitCount = 16;//16; //VIDEO_BIT_COUNT_YUV420; //16

    VideoMediaType.formattype = FORMAT_VideoInfo;
    VideoMediaType.pbFormat = (BYTE *)&VideoInfo;
    VideoMediaType.cbFormat = sizeof(VIDEOINFOHEADER);

    StillInfo = VideoInfo;
    StillInfo.bmiHeader.biWidth = StillWidth;
    StillInfo.bmiHeader.biHeight = StillHeight;
    StillMediaType = VideoMediaType;
    StillMediaType.pbFormat = (BYTE *)&StillInfo;

    PreviewInfo = VideoInfo;
    PreviewInfo.bmiHeader.biWidth = PreviewWidth;
    PreviewInfo.bmiHeader.biHeight = PreviewHeight;

    PreviewMediaType = VideoMediaType;
    PreviewMediaType.pbFormat = (BYTE *)&PreviewInfo;

    hr = CoCreateInstance(CLSID_NvVideoCapture, NULL, CLSCTX_INPROC_SERVER, IID_IBaseFilter, (void **)&m_pSourceFilter);
    FAILEDHR(hr, "CoCreateInstance for video capture filter failed.\n", CleanUp);

    // Video Capture filters supports the IKsPropertySet interface
    hr = m_pSourceFilter->QueryInterface(IID_IKsPropertySet, (void**)&m_pPropSetFilter);
    FAILEDHR(hr, "QueryInterface for INvSourceCfg failed for video.\n", CleanUp);

    hr = SetSourceFilterPinsMediaType(m_pSourceFilter, &PreviewMediaType, NULL, &StillMediaType);
    FAILEDHR(hr, "SetSourceFilterPinsMediaType failed.\n", CleanUp);

    hr = SetSourceFilterPinsMediaType(m_pSourceFilter, &PreviewMediaType, &VideoMediaType, NULL);
    FAILEDHR(hr, "SetSourceFilterPinsMediaType failed.\n", CleanUp);

    hr = pGraph->AddFilter(m_pSourceFilter, L"Video Capture Filter");
    FAILEDHR(hr, "AddFilter of Video capture failed.\n", CleanUp);

    hr = CoCreateInstance(
        CLSID_NvVideoEncoder,
        NULL,
        CLSCTX_INPROC_SERVER,
        IID_IBaseFilter,
        (void **)&m_pEncoderFilterStill);
    FAILEDHR(hr, "CoCreateInstance for Still Image encoder filter failed.\n", CleanUp); 

    hr = CoCreateInstance(
        CLSID_NvVideoEncoder,
        NULL,
        CLSCTX_INPROC_SERVER,
        IID_IBaseFilter,
        (void **)&m_pEncoderFilterVideo);
    FAILEDHR(hr, "CoCreateInstance for video encoder filter failed.\n", CleanUp); 

    hr = SetVideoPropertiesJPEG(m_pEncoderFilterStill);
    FAILEDHR(hr, "Unable to set still image encoder properties.\n", CleanUp);

    hr = SetVideoPropertiesH264(m_pEncoderFilterVideo);
    FAILEDHR(hr, "Unable to set H264 video encoder properties.\n", CleanUp);

    hr = pGraph->AddFilter(m_pEncoderFilterStill, L"Still Image Encoder Filter");
    FAILEDHR(hr, "AddFilter for Still Image encoder failed.\n", CleanUp);	 

    hr = pGraph->AddFilter(m_pEncoderFilterVideo, L"H264 Video Encoder Filter");
    FAILEDHR(hr, "AddFilter for H264 Video encoder failed.\n", CleanUp);	 

    // Create the video renderer
    hr = CoCreateInstance(CLSID_NvVideoRenderer, NULL, CLSCTX_INPROC_SERVER, IID_IBaseFilter, (void **)&m_pRendererFilter);
    FAILEDHR(hr, "CoCreateInstance of renderer filter failed.\n", CleanUp);

    // Add the video renderer filter
    hr = pGraph->AddFilter(m_pRendererFilter, L"Video Renderer Filter");
    FAILEDHR(hr, "Video renderer AddFilter failed.\n", CleanUp);

    //// Render the preview pin
    hr = pCaptureGraphBuilder->RenderStream(&PIN_CATEGORY_PREVIEW, &MEDIATYPE_Video, (IUnknown *)m_pSourceFilter, NULL, m_pRendererFilter);
    FAILEDHR(hr, "Render for preview failed.\n", CleanUp);

    // Render the capture pin
    hr = pCaptureGraphBuilder->RenderStream(&PIN_CATEGORY_CAPTURE, NULL, (IUnknown *)m_pSourceFilter, m_pEncoderFilterVideo, pSinkFilterVideo);
    FAILEDHR(hr, "Render for capture failed.\n", CleanUp);

    // Render the still pin
    hr = pCaptureGraphBuilder->RenderStream(&PIN_CATEGORY_STILL, NULL, (IUnknown *)m_pSourceFilter, m_pEncoderFilterStill, pSinkFilterStill);
    FAILEDHR(hr, "Render for still failed.\n", CleanUp);



CleanUp:
    if (m_pEncoderFilterStill) (void)m_pEncoderFilterStill->Release();
    if (m_pEncoderFilterVideo) (void)m_pEncoderFilterVideo->Release();    
    if (m_pPropSetFilter) (void)m_pPropSetFilter->Release();
    if (m_pSourceFilter) (void)m_pSourceFilter->Release();
    if (m_pRendererFilter) (void)m_pRendererFilter->Release();
    return hr;
}

HRESULT
RenderAudioInGraph(
                   IGraphBuilder *pGraph,
                   ICaptureGraphBuilder2 *pCaptureGraphBuilder,
                   IBaseFilter *pSinkFilter)
{
    IBaseFilter *pSourceFilter = NULL;
    IBaseFilter *pEncoderFilter = NULL;
    IAMStreamConfig *pStreamConfig = NULL;
    IEnumPins *pEnumPins = NULL;
    IPin *pPin = NULL;
    ULONG cFetched = 0;

    AM_MEDIA_TYPE AudioMediaType = {0};
    WAVEFORMATEX wfx = {0};
    HRESULT hr = E_FAIL;

    //NV_ASSERT(pGraph != NULL);
    //NV_ASSERT(pCaptureGraphBuilder != NULL);
    //NV_ASSERT(pSinkFilter != NULL);

    wfx.nChannels = 2; // Default is stereo
    wfx.nSamplesPerSec = 44100; // Defailt is 44.1 KHz

    AudioMediaType.formattype = FORMAT_WaveFormatEx;
    AudioMediaType.pbFormat = (BYTE *)&wfx;
    AudioMediaType.cbFormat = sizeof(WAVEFORMATEX);

    hr = CoCreateInstance(
        CLSID_NvAudioCapture,
        NULL,
        CLSCTX_INPROC_SERVER,
        IID_IBaseFilter,
        (void **)&pSourceFilter);
    FAILEDHR(hr, "CoCreateInstance for audio capture filter failed.\n", CleanUp);

    hr = pSourceFilter->EnumPins(&pEnumPins);
    FAILEDHR(hr, "EnumPins failed.\n", CleanUp);

    while (pEnumPins->Next(1, &pPin, &cFetched) == S_OK)
    {
        hr = pPin->QueryInterface(IID_IAMStreamConfig, (void **)&pStreamConfig);
        if (SUCCEEDED(hr))
            break;

        (void)pStreamConfig->Release();
        pStreamConfig = NULL;

        (void)pPin->Release();
        pPin = NULL;
    }

    if (pStreamConfig != NULL)
    {
        hr = pStreamConfig->SetFormat(&AudioMediaType);
        FAILEDHR(hr, "SetFormat failed.\n", CleanUp);
    }

    hr = pGraph->AddFilter(pSourceFilter, L"Audio Capture Filter");
    FAILEDHR(hr, "AddFilter of Audio capture failed.\n", CleanUp);

    hr = CoCreateInstance(
        CLSID_NvAudioEncoder,
        NULL,
        CLSCTX_INPROC_SERVER,
        IID_IBaseFilter,
        (void **)&pEncoderFilter);
    FAILEDHR(hr, "CoCreateInstance for audio encoder filter failed.\n", CleanUp);

    hr = SetAudioProperties(pEncoderFilter);
    FAILEDHR(hr, "Unable to set audio encoder properties.\n", CleanUp);

    hr = pGraph->AddFilter(pEncoderFilter, L"Audio Encoder Filter");
    FAILEDHR(hr, "AddFilter for audio encoder failed.\n", CleanUp);

    hr = pCaptureGraphBuilder->RenderStream(&PIN_CATEGORY_CAPTURE, NULL, (IUnknown *)pSourceFilter, pEncoderFilter, pSinkFilter);
    FAILEDHR(hr, "Render for audio capture failed.\n", CleanUp);

CleanUp:
    if (pEncoderFilter) (void)pEncoderFilter->Release();
    if (pSourceFilter) (void)pSourceFilter->Release();
    if (pStreamConfig) (void)pStreamConfig->Release();
    if (pEnumPins) (void)pEnumPins->Release();
    if (pPin) (void)pPin->Release();
    return hr;
}

HRESULT AddSinkFilter(IGraphBuilder *pGraph, IBaseFilter **ppSinkFilter, BOOL bIsStill)
{
    IBaseFilter *pWriterFilter = NULL;
    WCHAR FileNameW[256] = {0};
    IKsPropertySet *pMuxPropSet = NULL;
    //NvU64 MaxFileSize;
    HRESULT hr = E_FAIL;

    //NV_ASSERT(pGraph != NULL);
    //NV_ASSERT(ppSinkFilter != NULL);

    hr = CoCreateInstance(
        CLSID_Nv3GP2MuxFilter, // Mux is also the file writer
        NULL,
        CLSCTX_INPROC_SERVER,
        IID_IBaseFilter,
        (void **)&pWriterFilter);
    FAILEDHR(hr, "CoCreateInstance of mux filter failed.\n", CleanUp);

    hr = pWriterFilter->QueryInterface(IID_IFileSinkFilter, (void**)(&m_pFileSinkFilter));
    FAILEDHR(hr, "QueryInterface of mux filter failed.\n", CleanUp);

    hr = pWriterFilter->QueryInterface(IID_IKsPropertySet,(void**)(&pMuxPropSet));
    FAILEDHR(hr, "QueryInterface of mux filter failed.\n", CleanUp);

    mbstowcs_s(NULL, FileNameW, sizeof(FileNameW), bIsStill ? g_outFileStillName : g_OutFilename, 256);	

    hr = m_pFileSinkFilter->SetFileName(FileNameW, NULL);
    FAILEDHR(hr, "SetFileName of mux filter failed.\n", CleanUp);

    hr = pGraph->AddFilter(pWriterFilter, bIsStill ? L"Still Image File Writer" : L"Video File Writer");
    FAILEDHR(hr, "AddFilter of dump filter failed.\n", CleanUp);

    //TODO	
    //MaxFileSize = 200000000; // Default maximum file size is 200 MB
    //hr = pMuxPropSet->Set(PROPSET_NvFilter,
    //                    NvPropertyId_EncodedFileSize,
    //                    NULL,
    //                    0,
    //                    &MaxFileSize,
    //                    sizeof(NvU64));
    //FAILEDHR(hr, "Write method of MuxFilter::IPropertyBag for MaxFileSize failed.\n", CleanUp);

    hr = S_OK;

CleanUp:
    if (m_pFileSinkFilter) (void)m_pFileSinkFilter->Release();
    if (pMuxPropSet) (void)pMuxPropSet->Release();

    if (pWriterFilter) 
    {
        if (SUCCEEDED(hr))
            *ppSinkFilter = pWriterFilter;
        else
            (void)pWriterFilter->Release();
    }

    return hr;
}

/*
HRESULT SetFilterProperty(
    IBaseFilter *pFilter,
    REFGUID guidPropSet,
    DWORD dwPropID,
    void *pInstanceData,
    DWORD cbInstanceData,
    void *pPropData,
    DWORD cbPropData
    )
{
    IKsPropertySet *pMuxPropSet = NULL;
    HRESULT hr = E_FAIL;

    hr = pFilter->QueryInterface(IID_IKsPropertySet,(void**)(&pMuxPropSet));
    FAILEDHR(hr, "QueryInterface of mux filter failed.\n", CleanUp);

    hr = pMuxPropSet->Set(guidPropSet,
                        dwPropID,
                        pInstanceData,
                        cbInstanceData,
                        pPropData,
                        cbPropData);
    FAILEDHR(hr, "Write method of MuxFilter::IPropertyBag for MaxFileSize failed.\n", CleanUp);

    hr = S_OK;

CleanUp:
    if (pMuxPropSet) (void)pMuxPropSet->Release();
    return hr;
}
*/

HRESULT StopStreamCapture()
{
    HRESULT hr = S_OK;
    //WORD wAudioStartCookie = 7, wAudioStopCookie = 8;
    //IMediaSeeking *pMediaSeeking;
    WCHAR FileNameW[256] = {0};

    /*if(m_pGraph)
    {
    hr = m_pGraph->QueryInterface( &pMediaSeeking );
    FAILEDHR(hr, "CCaptureFramework: Retrieving the MediaSeeking interface failed.", CleanUp);

    hr = pMediaSeeking->GetCurrentPosition(&rtStop);
    FAILEDHR(hr, "CCaptureFramework: Failed to retrieve the current position.", CleanUp);

    }*/

    if(m_pSourceFilter)
    {
        REFERENCE_TIME rtStart = MAXLONGLONG;
        REFERENCE_TIME rtStop = 0;
        WORD wVideoStartCookie = 5, wVideoStopCookie = 6;
        hr = m_pCaptureGraphBuilder->ControlStream( &PIN_CATEGORY_CAPTURE, &MEDIATYPE_Video, m_pSourceFilter, &rtStart, &rtStop, wVideoStartCookie, wVideoStopCookie);
        FAILEDHR(hr,"CCaptureFramework: Failed to stop the video capture.", CleanUp);
        BYTE bVal = 1;
        /*SetFilterProperty(m_pSinkFilterVideo, PROPSET_NvFilter,
            NvPropertyId_MuxWriterReinitializeFilename, NULL, 0, &bVal, sizeof(bVal));*/

        m_pSinkFilterVideo->Stop();
        m_pEncoderFilterVideo->Stop();
        m_pSinkFilterVideo->Run(0);
        m_pEncoderFilterVideo->Run(0);

    }
	
	if(g_AudioEnable)
    {
        hr = m_pControl->Stop();
    
        m_pAudEncoderFilter->Release();
        m_pAudSourceFilter->Release();

        m_pGraph->RemoveFilter(m_pAudEncoderFilter);
        m_pGraph->RemoveFilter(m_pAudSourceFilter);
  
        hr = m_pControl->Run();	
    }


#if 0
    //Fix for more no of video capture, temporary file not deleting and next video capture is not occuring
    //if(g_StillEnable || g_AudioEnable)
    {
        //change to still Encoder filer
        hr = m_pControl->Stop();
    }

    /*if(g_StillEnable)
    {		
    m_pEncoderFilter->Release();
    m_pGraph->RemoveFilter(m_pEncoderFilter);
    }*/

    if(g_AudioEnable)
    {
        m_pAudEncoderFilter->Release();
        m_pAudSourceFilter->Release();

        m_pGraph->RemoveFilter(m_pAudEncoderFilter);
        m_pGraph->RemoveFilter(m_pAudSourceFilter);
    }

    /*if(g_StillEnable)
    {
    hr = CoCreateInstance(
    CLSID_NvVideoEncoder,
    NULL,
    CLSCTX_INPROC_SERVER,
    IID_IBaseFilter,
    (void **)&m_pEncoderFilter);
    FAILEDHR(hr, "CoCreateInstance for video encoder filter failed.\n", CleanUp); 

    hr = SetVideoPropertiesJPEG(m_pEncoderFilter);
    FAILEDHR(hr, "Unable to set video encoder properties.\n", CleanUp);

    hr = m_pGraph->AddFilter(m_pEncoderFilter, L"Video Encoder Filter");
    FAILEDHR(hr, "AddFilter for video encoder failed.\n", CleanUp);

    mbstowcs_s(NULL, FileNameW, sizeof(FileNameW), g_outFileStillName, 256);
    hr = m_pFileSinkFilter->SetFileName(FileNameW, NULL);	

    hr = m_pCaptureGraphBuilder->RenderStream(&PIN_CATEGORY_STILL, NULL, (IUnknown *)m_pSourceFilter, m_pEncoderFilter, m_pSinkFilter);
    FAILEDHR(hr, "Render for still failed.\n", CleanUp);		
    }*/

    //if(g_StillEnable || g_AudioEnable)
    {
        hr = m_pControl->Run();	
    }
#endif
CleanUp:
    return hr;
}



HRESULT StartStreamCapture()
{
    HRESULT hr = S_OK;
    //WORD wAudioStartCookie = 3, wAudioStopCookie = 4;
    WCHAR FileNameW[256] = {0};

    IAMStreamConfig *pStreamConfig = NULL;
    IEnumPins *pEnumPins = NULL;
    IPin *pPin = NULL;
    ULONG cFetched = 0;
    AM_MEDIA_TYPE AudioMediaType = {0};
    WAVEFORMATEX wfx = {0};
    
    /*if(g_StillEnable)
    {		
    m_pGraph->RemoveFilter(m_pEncoderFilter);	 
    m_pEncoderFilter->Release();

    hr = CoCreateInstance(
    CLSID_NvVideoEncoder,
    NULL,
    CLSCTX_INPROC_SERVER,
    IID_IBaseFilter,
    (void **)&m_pEncoderFilter);
    FAILEDHR(hr, "CoCreateInstance for video encoder filter failed.\n", CleanUp); 

    hr = SetVideoPropertiesH264(m_pEncoderFilter);
    FAILEDHR(hr, "Unable to set video encoder properties.\n", CleanUp);

    hr = m_pGraph->AddFilter(m_pEncoderFilter, L"Video Encoder Filter");
    FAILEDHR(hr, "AddFilter for video encoder failed.\n", CleanUp);

    mbstowcs_s(NULL, FileNameW, sizeof(FileNameW), g_OutFilename, 256);
    hr = m_pFileSinkFilter->SetFileName(FileNameW, NULL);	

    hr = m_pCaptureGraphBuilder->RenderStream(&PIN_CATEGORY_CAPTURE, NULL, (IUnknown *)m_pSourceFilter, m_pEncoderFilter, m_pSinkFilter);
    FAILEDHR(hr, "Render for capture failed.\n", CleanUp);
    }*/
	
	if(g_AudioEnable)
    {
        hr = m_pControl->Stop();

        wfx.nChannels = 2; // Default is stereo
        wfx.nSamplesPerSec = 44100; // Defailt is 44.1 KHz

        AudioMediaType.formattype = FORMAT_WaveFormatEx;
        AudioMediaType.pbFormat = (BYTE *)&wfx;
        AudioMediaType.cbFormat = sizeof(WAVEFORMATEX);

        hr = CoCreateInstance(
            CLSID_NvAudioCapture,
            NULL,
            CLSCTX_INPROC_SERVER,
            IID_IBaseFilter,
            (void **)&m_pAudSourceFilter);
        FAILEDHR(hr, "CoCreateInstance for audio capture filter failed.\n", CleanUp);

        hr = m_pAudSourceFilter->EnumPins(&pEnumPins);
        FAILEDHR(hr, "EnumPins failed.\n", CleanUp);

        while (pEnumPins->Next(1, &pPin, &cFetched) == S_OK)
        {
            hr = pPin->QueryInterface(IID_IAMStreamConfig, (void **)&pStreamConfig);
            if (SUCCEEDED(hr))
                break;

            (void)pStreamConfig->Release();
            pStreamConfig = NULL;

            (void)pPin->Release();
            pPin = NULL;
        }

        if (pStreamConfig != NULL)
        {
            hr = pStreamConfig->SetFormat(&AudioMediaType);
            FAILEDHR(hr, "SetFormat failed.\n", CleanUp);
        }

        hr = m_pGraph->AddFilter(m_pAudSourceFilter, L"Audio Capture Filter");
        FAILEDHR(hr, "AddFilter of Audio capture failed.\n", CleanUp);

        hr = CoCreateInstance(
            CLSID_NvAudioEncoder,
            NULL,
            CLSCTX_INPROC_SERVER,
            IID_IBaseFilter,
            (void **)&m_pAudEncoderFilter);
        FAILEDHR(hr, "CoCreateInstance for audio encoder filter failed.\n", CleanUp);

        hr = SetAudioProperties(m_pAudEncoderFilter);
        FAILEDHR(hr, "Unable to set audio encoder properties.\n", CleanUp);

        hr = m_pGraph->AddFilter(m_pAudEncoderFilter, L"Audio Encoder Filter");
        FAILEDHR(hr, "AddFilter for audio encoder failed.\n", CleanUp);

        hr = m_pCaptureGraphBuilder->RenderStream(&PIN_CATEGORY_CAPTURE, NULL, (IUnknown *)m_pAudSourceFilter, m_pAudEncoderFilter, m_pSinkFilterVideo);
        FAILEDHR(hr, "Render for audio capture failed.\n", CleanUp);
    
		hr = m_pControl->Run(); 	
    }

    if(m_pSourceFilter)
    {
        /* hr = ControlCaptureFilterPin(
        m_pGraph,
        PIN_CATEGORY_CAPTURE,
        &rtStart,
        &rtStop);
        FAILEDHR(hr, "ControlCaptureFilterPin failed.\n", CleanUp);  */  
        REFERENCE_TIME rtStart = 0;
        REFERENCE_TIME rtStop = MAXLONGLONG;
        WORD wVideoStartCookie = 1, wVideoStopCookie = 2;
        hr = m_pCaptureGraphBuilder->ControlStream( &PIN_CATEGORY_CAPTURE, &MEDIATYPE_Video, m_pSourceFilter, &rtStart, &rtStop, wVideoStartCookie, wVideoStopCookie );
        FAILEDHR(hr, "CCaptureFramework: Failed to start the video capture.", CleanUp);        
    }

CleanUp:
    return hr;
}

HRESULT RunGraph(HWND ghApp, IGraphBuilder *pGraph)
{
    MSG msg={0};
    BOOL bKill = FALSE;
    int ret = 0;
    HRESULT hr = S_OK;
    IMediaEvent *pEvent = NULL;

    // to interact with the videoWindow, we have to create one and tie the video output into it.
    IVideoWindow  * g_pVidWin = NULL;

    hr = pGraph->QueryInterface(IID_IMediaControl, (void**)&m_pControl);
    FAILEDHR(hr, "QueryInterface for IMediaControl failed.\n", CleanUp);

    hr = pGraph->QueryInterface(IID_IMediaEvent, (void**)&m_pEvent);
    FAILEDHR(hr, "QueryInterface for IMediaControl failed.\n", CleanUp);

    hr = pGraph->QueryInterface(IID_IMediaEventEx, (void**)&m_pEventEx);
    FAILEDHR(hr, "QueryInterface for IMediaControl failed.\n", CleanUp);

    if(g_VideoWindow)
    {
        hr = pGraph->QueryInterface (IID_IVideoWindow, (LPVOID *) &g_pVidWin);
        if (FAILED (hr))
        {
            MessageBox (ghApp, _T("unable to obtain IVideoWindow interface.\n"),_T("Error"), MB_OK);
            return hr;
        } 
        else 
        {
            g_videoWindow = g_pVidWin;
        }

        // tell VideoWindow to forward messages to our Window.
        g_videoWindow->put_MessageDrain((OAHWND) ghApp);
        SetupVideoWindow();
        ShowWindow(ghApp, SW_SHOWNORMAL);
    }

    // Since ICaptureGraphBuilder::ControlStream is not available on CE5
    // We had to implement this function of control individual streams    	
    /* hr = ControlCaptureFilterPin(
    m_pGraph,
    PIN_CATEGORY_CAPTURE,
    &CaptureStartDelay,
    &CaptureStop);
    FAILEDHR(hr, "ControlCaptureFilterPin failed.\n", CleanUp);    */

    WORD wVideoStartCookie = 1, wVideoStopCookie = 2;
    REFERENCE_TIME CaptureStartDelay = MAXLONGLONG;
    REFERENCE_TIME CaptureStop = 0;
    m_pCaptureGraphBuilder->ControlStream( &PIN_CATEGORY_CAPTURE, &MEDIATYPE_Video, m_pSourceFilter, &CaptureStartDelay, &CaptureStop, wVideoStartCookie, wVideoStopCookie );


    DumpGraphEx(pGraph);
    hr = m_pControl->Run();
    if (hr == S_FALSE) {
        // graph tries to run, but takes some time....
        OAFilterState filterState;
        hr = m_pControl->GetState(0, &filterState);
        Sleep(1);
    }
    FAILEDHR(hr, "Run failed (%x).\n", CleanUp);

    //hr = pEvent->WaitForCompletion(Duration, &EventCode);
    if(g_VideoWindow)
    {

        // Main message loop
        while(GetMessage(&msg,NULL,0,0))
        {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
    }
    else
    {
        while(!bKill)
        {	
            ret = getchar();
            switch(ret) 
            {
            case 'c':
            case 'C':
                StartStreamCapture();
                break;

            case 'x':
            case 'X':
                StopStreamCapture();
                break;

            case 's':
            case 'S':
                TriggerPin(PIN_CATEGORY_STILL);
                break;

            case 'e':
            case 'E':
                //StopStreamCapture();
                bKill = TRUE;
                break;			
            }
        }
    }
    if (FAILED(hr) && hr != E_ABORT)
    {
        printf("WaitForCompletion failed.\n");
        goto CleanUp;
    }

    hr = m_pControl->Stop();
    //NV_ASSERT(hr == S_OK);

    if(g_VideoWindow)
    {
        if (g_pVidWin)
        {
            g_pVidWin->put_Visible(OAFALSE);
            g_pVidWin->put_Owner(NULL);
        }
    }


CleanUp:
    if (m_pControl)
        (void)m_pControl->Release();

    if (pEvent)
        (void)pEvent->Release();

    if (m_pEventEx)
        (void)m_pEventEx->Release();

    if(g_VideoWindow)
    {
        if (g_pVidWin)
            (void)g_pVidWin->Release();
    }

    return hr;
}

#ifndef EC_STATE_CHANGE
#define EC_STATE_CHANGE 0x32
#endif

HRESULT HandleGraphEvent(void)
{
    LONG evCode;
    LONG_PTR evParam1, evParam2;
    HRESULT hr=S_OK;

    if (!m_pEventEx)
        return E_POINTER;

    while(SUCCEEDED(m_pEventEx->GetEvent(&evCode, &evParam1, &evParam2, 0)))
    {
        switch(evCode)
        {
        case EC_COMPLETE:
            RETAILMSG(1, (TEXT("GraphEvent: EC_COMPLETE: HRESULT=%08lx\r\n"), evParam1));
            break;
        case EC_STATE_CHANGE:
            RETAILMSG(1, (TEXT("GraphEvent: EC_STATE_CHANGE: %s\r\n"), evParam1==0 ? TEXT("Stopped") : evParam1==1 ? TEXT("Paused") : TEXT("Running") ));
            break;
        case EC_VIDEO_SIZE_CHANGED:
            RETAILMSG(1, (TEXT("GraphEvent: EC_VIDEO_SIZE_CHANGED: %d x %d\r\n"),  LOWORD(evParam1), HIWORD(evParam1) ));
            break;
        case EC_VIDEOFRAMEREADY:
            RETAILMSG(1, (TEXT("GraphEvent: EC_VIDEOFRAMEREADY\r\n") ));
            break;
        case EC_ERRORABORT:
            RETAILMSG(1, (TEXT("GraphEvent: EC_ERRORABORT: HRESULT=%08lx\r\n"), evParam1));
            break;
        case EC_CLOCK_CHANGED:
            RETAILMSG(1, (TEXT("GraphEvent: EC_CLOCK_CHANGED\r\n") ));
            break;
        case EC_STREAM_CONTROL_STARTED:
            RETAILMSG(1, (TEXT("GraphEvent: EC_STREAM_CONTROL_STARTED: pSender=%08lx, dwCookie=%d\r\n"), evParam1, evParam2));
            break;
        case EC_STREAM_CONTROL_STOPPED:
            RETAILMSG(1, (TEXT("GraphEvent: EC_STREAM_CONTROL_STOPPED: pSender=%08lx, dwCookie=%d\r\n"), evParam1, evParam2));
            break;
        default:
            RETAILMSG(1, (TEXT("Unexpected Event(%08lx) in HandleGraphEvent() !\r\n"), evCode));
            break;
        }

        //
        // Free event parameters to prevent memory leaks associated with
        // event parameter data.  While this application is not interested
        // in the received events, applications should always process them.
        //
        hr = m_pEventEx->FreeEventParams(evCode, evParam1, evParam2);
    }
    return hr;
}

#if 0 // only needed for old ce versions (CE5)
HRESULT
ControlCaptureFilterPin(
                        IGraphBuilder *pGraph,
                        const GUID PinCategory,
                        REFERENCE_TIME *pStart,
                        REFERENCE_TIME *pStop)
{
    IEnumFilters *pEnumFilters = NULL;
    IBaseFilter *pFilter = NULL;
    IEnumPins *pEnumPins = NULL;
    IPin *pPin = NULL;
    IAMStreamControl *pStreamControl = NULL;
    IAMFilterMiscFlags *pMiscFlags = NULL;
    IKsPropertySet *pKsPropertySet = NULL;
    ULONG cFetched = 0;
    HRESULT hr = S_OK;
    GUID CurrentPinCategory = GUID_NULL;

    //NV_ASSERT(pGraph != NULL);

    hr = pGraph->EnumFilters(&pEnumFilters);
    FAILEDHR(hr, "ControlCaptureFilterPin: EnumFilters failed.\n", CleanUp);

    while (pEnumFilters->Next(1, &pFilter, &cFetched) == S_OK)
    {
        hr = pFilter->QueryInterface(IID_IAMFilterMiscFlags, (void **)&pMiscFlags);
        if (FAILED(hr))
        {
            (void)pFilter->Release();
            pFilter = NULL;
            continue;
        }

        if (pMiscFlags->GetMiscFlags() != AM_FILTER_MISC_FLAGS_IS_SOURCE)
        {
            (void)pMiscFlags->Release();
            pMiscFlags = NULL;
            (void)pFilter->Release();
            pFilter = NULL;
            continue;
        }

        (void)pMiscFlags->Release();
        pMiscFlags = NULL;

        hr = pFilter->EnumPins(&pEnumPins);
        FAILEDHR(hr, "ControlCaptureFilterPin: EnumPins failed.\n", CleanUp);

        while (pEnumPins->Next(1, &pPin, &cFetched) == S_OK)
        {
            hr = pPin->QueryInterface(IID_IKsPropertySet, (void **)&pKsPropertySet);
            FAILEDHR(hr, "ControlCaptureFilterPin: QueryInterface for IKsPropertySet failed.\n", CleanUp);

            if (IsEqualGUID(CurrentPinCategory, PinCategory))
            {
                hr = pPin->QueryInterface(IID_IAMStreamControl, (void **)&pStreamControl);
                FAILEDHR(hr, "ControlCaptureFilterPin: QueryInterface of stream control failed.\n", CleanUp);

                (void)pStreamControl->StartAt(pStart, 0);
                (void)pStreamControl->StopAt(pStop, FALSE, 0);
                printf("Setting StreamControl to start at %li.\n", pStart);

                (void)pStreamControl->Release();
                pStreamControl = NULL;
            }

            (void)pPin->Release();
            pPin = NULL;

            (void)pKsPropertySet->Release();
            pKsPropertySet = NULL;
        }

        (void)pEnumPins->Release();
        pEnumPins = NULL;

        (void)pFilter->Release();
        pFilter = NULL;
    }

CleanUp:
    if (pEnumFilters)
        (void)pEnumFilters->Release();

    if (pFilter)
        (void)pFilter->Release();

    if (pEnumPins)
        (void)pEnumPins->Release();

    if (pPin)
        (void)pPin->Release();

    if (pStreamControl)
        (void)pStreamControl->Release();

    if (pMiscFlags)
        (void)pMiscFlags->Release();

    if (pKsPropertySet)
        (void)pKsPropertySet->Release();

    return hr;
}
#endif

HRESULT SetVideoPropertiesH264(IBaseFilter *pEncoderFilter)
{
    IPropertyBag *pPropertyBag = NULL;
    HRESULT hr = E_FAIL;
    VARIANT var;

    const WCHAR *FormatTypeW = L"NVVIDEO";
    const WCHAR *AudioOnlyW = L"_YES";
    const WCHAR *MaxBitRateW = L"MAX_BIT_RATE";
    const WCHAR *VideoAppTypeW = L"VIDEO_APP_TYPE";
    const WCHAR *VideoQualityLevelW = L"VIDEO_QUALITY_LEVEL";
    const WCHAR *VideoErrorResiliencyLevelW = L"VIDEO_ERROR_RESILIENCY_LEVEL";

    //NV_ASSERT(pEncoderFilter != NULL);

    hr = pEncoderFilter->QueryInterface(IID_IPropertyBag, (void**)&pPropertyBag);
    FAILEDHR(hr, "QueryInterface for IPropertyBag failed.\n", CleanUp);

    VariantInit(&var);
    var.vt = VT_I4;
    var.lVal = 100; //NvVideoCodecType_H264; // Default video encoder type is H.264
    hr = pPropertyBag->Write(FormatTypeW, &var);
    VariantClear(&var);
    FAILEDHR(hr, "Write method of IPropertyBag for VideoEncoderType failed.\n", CleanUp);

    VariantInit(&var);
    var.vt = VT_I4;
    var.lVal = STREAM_NO; // Default case is audio+video, so AudioOnly is NO
    hr = pPropertyBag->Write(AudioOnlyW, &var);
    VariantClear(&var);
    FAILEDHR(hr, "Write method of IPropertyBag for AudioOnly flag failed.\n", CleanUp);

    VariantInit(&var);
    var.vt = VT_I4;
    var.lVal = 1;//NvAppType_Camcorder; // Default use-case is camcorder
    hr = pPropertyBag->Write(VideoAppTypeW, &var);
    VariantClear(&var);
    FAILEDHR(hr, "Write method of IPropertyBag for VideoAppType failed.\n", CleanUp);

    VariantInit(&var);
    var.vt = VT_I4;
    var.lVal = 2;//NvQualityLevel_Medium; // Default video quality level is medium
    hr = pPropertyBag->Write(VideoQualityLevelW, &var);
    VariantClear(&var);
    FAILEDHR(hr, "Write method of IPropertyBag for VideoQualityLevel failed.\n", CleanUp);

    VariantInit(&var);
    var.vt = VT_I4;
    var.lVal = 1;//NvErrorResiliencyLevel_None; // Default resiliency level is none
    hr = pPropertyBag->Write(VideoErrorResiliencyLevelW, &var);
    VariantClear(&var);
    FAILEDHR(hr, "Write method of IPropertyBag for VideoErrorResiliencyLevel failed.\n", CleanUp);

    VariantInit(&var);
    var.vt = VT_I4;
    var.lVal = DEFAULT_BITRATE; // Default bitrate is 10 MBps
    hr = pPropertyBag->Write(MaxBitRateW, &var);
    VariantClear(&var);
    FAILEDHR(hr,"Write method of IPropertyBag for VideoBitRate failed.\n",CleanUp);

    hr = S_OK;

CleanUp:
    if (pPropertyBag) (void)pPropertyBag->Release();
    return hr;
}

HRESULT SetVideoPropertiesJPEG(IBaseFilter *pEncoderFilter)
{
    IPropertyBag *pPropertyBag = NULL;
    HRESULT hr = E_FAIL;
    VARIANT var;

    const WCHAR *FormatTypeW = L"NVVIDEO";
    const WCHAR *ENCODER_JPEG_QUALITY_LEVEL = L"ENCODER_JPEG_QUALITY_LEVEL";


    hr = pEncoderFilter->QueryInterface(IID_IPropertyBag, (void**)&pPropertyBag);
    FAILEDHR(hr, "QueryInterface for IPropertyBag failed.\n", CleanUp);

    VariantInit(&var);
    var.vt = VT_I4;
    var.lVal = 103; // NvVideoCodecType_JPEG; 
    hr = pPropertyBag->Write(FormatTypeW, &var);
    FAILEDHR(hr, "Write method of IPropertyBag for VideoEncoderType failed.\n", CleanUp);
    VariantClear(&var);

    VariantInit(&var);
    var.vt = VT_I4;
    var.lVal = 5;	// NvJpegQualityLevel_VeryHigh
    hr = pPropertyBag->Write(ENCODER_JPEG_QUALITY_LEVEL, &var);
    FAILEDHR(hr, "Write method of IPropertyBag for VideoEncoderType failed.\n", CleanUp);
    VariantClear(&var);
    hr = S_OK;

CleanUp:
    if (pPropertyBag)        (void)pPropertyBag->Release();
    return hr;
}

HRESULT SetAudioProperties(IBaseFilter *pEncoderFilter)
{
    IPropertyBag *pPropertyBag = NULL;
    HRESULT hr = E_FAIL;
    VARIANT var;

    const WCHAR *FormatTypeW = L"NVAUDIO";
    const WCHAR *SpeechModeW = L"AMR_SPEECH_MODE"; // Audio bit-rate (for all formats)
    const WCHAR *VideoOnlyW = L"_YES";

    //NV_ASSERT(pEncoderFilter != NULL);

    hr = pEncoderFilter->QueryInterface(IID_IPropertyBag, (void**)&pPropertyBag);
    FAILEDHR(hr, "QueryInterface for IPropertyBag failed.\n", CleanUp);

    VariantInit(&var);
    var.vt = VT_I4;
    var.lVal = 4;// NvAudioCodecType_AAC; // Default audio encoder type is AACLC
    hr = pPropertyBag->Write(FormatTypeW, &var);
    VariantClear(&var);
    FAILEDHR(hr, "Write method of IPropertyBag for AudioEncoderType failed.\n", CleanUp);

    VariantInit(&var);
    var.vt = VT_I4;
    var.lVal = 128000; // Default audio bit-rate is 128 kbps
    hr = pPropertyBag->Write(SpeechModeW, &var);
    VariantClear(&var);
    FAILEDHR(hr, "Write method of IPropertyBag for AudioMode failed.\n", CleanUp);

    VariantInit(&var);
    var.vt = VT_I4;
    var.lVal = STREAM_NO; // Default case is audio+video, so VideoOnly is NO
    hr = pPropertyBag->Write(VideoOnlyW, &var);
    VariantClear(&var);
    FAILEDHR(hr, "Write method of IPropertyBag for VideoOnly flag failed.\n", CleanUp);

    hr = S_OK;

CleanUp:
    if (pPropertyBag)
        (void)pPropertyBag->Release();

    return hr;
}

HRESULT SetSourceFilterPinsMediaType(IBaseFilter *pSourceFilter, AM_MEDIA_TYPE *pPreviewMediaType, AM_MEDIA_TYPE *pCaptureMediaType,AM_MEDIA_TYPE *pStillMediaType )
{
    IEnumPins *pEnumPins = NULL;
    IKsPropertySet *pKsPropertySet = NULL;
    IAMStreamConfig *pStreamConfig = NULL;
    HRESULT hr = S_OK;
    AM_MEDIA_TYPE *pCurrentMediaType = NULL;
    IPin *pPin = NULL;
    ULONG cFetched = 0;
    GUID CurrentPinCategory = GUID_NULL;
    DWORD   Returned = 0;

    hr = pSourceFilter->EnumPins(&pEnumPins);
    FAILEDHR(hr, "EnumPins failed.\n", CleanUp);

    while (pEnumPins->Next(1, &pPin, &cFetched) == S_OK)
    {
        hr = pPin->QueryInterface(IID_IKsPropertySet, (void **)&pKsPropertySet);
        FAILEDHR(hr, "QueryInterface for IKsPropertySet failed.\n", CleanUp);

        hr = pPin->QueryInterface(IID_IAMStreamConfig, (void **)&pStreamConfig);
        FAILEDHR(hr, "QueryInterface for IAMStreamConfig failed.\n", CleanUp);

        Returned = 0;

        hr = pKsPropertySet->Get(AMPROPSETID_Pin, AMPROPERTY_PIN_CATEGORY, NULL, 0, &CurrentPinCategory, sizeof(GUID), &Returned);
        FAILEDHR(hr, "KsPropertySet get for pin category failed.\n", CleanUp);

        if (IsEqualGUID(CurrentPinCategory, PIN_CATEGORY_PREVIEW))
            pCurrentMediaType = pPreviewMediaType;
        else if (IsEqualGUID(CurrentPinCategory, PIN_CATEGORY_CAPTURE))
            pCurrentMediaType = pCaptureMediaType;
        else if (IsEqualGUID(CurrentPinCategory, PIN_CATEGORY_STILL))
            pCurrentMediaType = pStillMediaType;
        else
            pCurrentMediaType = 0;

        if (pCurrentMediaType)
        {
            hr = pStreamConfig->SetFormat(pCurrentMediaType);
            FAILEDHR(hr, "KsPropertySet set for pin category failed.\n", CleanUp);
        }

        (void)pPin->Release();
        pPin = NULL;

        (void)pKsPropertySet->Release();
        pKsPropertySet = NULL;

        (void)pStreamConfig->Release();
        pStreamConfig = NULL;
    }

CleanUp:
    if (pEnumPins)
        (void)pEnumPins->Release();

    if (pKsPropertySet)
        (void)pKsPropertySet->Release();

    if (pPin)
        (void)pPin->Release();

    if (pStreamConfig)
        (void)pStreamConfig->Release();

    return hr;
}

HRESULT FindPinByCategory(IBaseFilter *filter, GUID pinCategory, IPin **ppPin)
{
    HRESULT hr = S_OK;
    IEnumPins *pEnumPins = NULL;
    IPin *pPin = NULL;
    ULONG cFetched = 0;
    GUID CurrentPinCategory = GUID_NULL;
    IKsPropertySet *pKsPropertySet = NULL;
    DWORD Returned = 0;

    hr = filter->EnumPins(&pEnumPins);																									  
    FAILEDHR(hr, "FindPinByCategory: EnumPins failed.\n", CleanUp);
    hr = pEnumPins->Next(1,&pPin,&cFetched);
    while (hr == S_OK) {
        hr = pPin->QueryInterface(IID_IKsPropertySet, (void **)&pKsPropertySet);
        FAILEDHR(hr, "FindPinByCategory: QueryInterface for IKsPropertySet failed.\n", CleanUp);
        Returned = 0;
        hr = pKsPropertySet->Get(AMPROPSETID_Pin, AMPROPERTY_PIN_CATEGORY, NULL, 0, &CurrentPinCategory, sizeof(GUID), &Returned);
        FAILEDHR(hr, "FindPinByCategory: IKsPropertySet get for pin category failed.\n", CleanUp);
        if (IsEqualGUID(CurrentPinCategory,pinCategory)) {
            if (ppPin != NULL) {
                *ppPin = pPin;
            }
            hr = S_OK;
            goto CleanUp;
        }
        if (pPin)  (void)pPin->Release();
        pPin = NULL;
        if (pKsPropertySet) (void)pKsPropertySet->Release();
        pKsPropertySet = NULL;
        hr = pEnumPins->Next(1,&pPin,&cFetched);
    }

CleanUp:
    if ((hr != S_OK) && pPin) (void)pPin->Release();
    if (pKsPropertySet) (void)pKsPropertySet->Release();
    if (pEnumPins) (void)pEnumPins->Release();
    return hr;
}

HRESULT  TriggerPin(GUID PinCategory )
{
    IBaseFilter *pSourceFilter = NULL;
    IAMVideoControl *pAMVidControl = NULL;
    HRESULT hr = S_OK;
    IPin *pPin = 0;

    //printf("Trying to trigger Pin \"%s\".\n", IsEqualGUID(PIN_CATEGORY_CAPTURE, PinCategory)?"Capture":IsEqualGUID(PIN_CATEGORY_STILL, PinCategory)?"Still":"Unexpected");
    if(m_pGraph!=NULL){   
        hr = m_pGraph->FindFilterByName(L"Video Capture Filter", &pSourceFilter);
        FAILEDHR(hr, "TriggerPin: FindFilterByName failed.\n", CleanUp);
        hr = pSourceFilter->QueryInterface(IID_IAMVideoControl, (void**) &pAMVidControl);   
        FAILEDHR(hr, "TriggerPin: QueryInterface on pFilter failed.\n", CleanUp);
        hr = FindPinByCategory(pSourceFilter,PinCategory,&pPin);
        FAILEDHR(hr, "TriggerPin: FindPinByCategory failed.\n", CleanUp);
        hr = pAMVidControl->SetMode(pPin, VideoControlFlag_Trigger);
        FAILEDHR(hr, "TriggerPin: SetMode failed.\n", CleanUp);
    }

CleanUp:
    if (pPin)				   (void)pPin->Release();
    if (pAMVidControl)         (void)pAMVidControl->Release();
    if (pSourceFilter)         (void)pSourceFilter->Release();
    return hr;
}

