#include "..\CameraPlugin\imager_hal.h"

#include <windows.h>
#include <commctrl.h>

#include "nvencode_test.h"
#include "resource.h"

#define WM_GRAPHNOTIFY		WM_APP+1

HWND ghApp;
//HWND ghCB=0;
IVideoWindow  *g_videoWindow = NULL;
HIMAGELIST g_hImageList = NULL;
static HWND s_hWndToolbar;
static DWORD TB_Height=0;

extern IMediaEventEx *m_pEventEx;
extern BOOL g_StillEnable;
extern IMediaControl *m_pControl;

HRESULT HandleGraphEvent(void);
HRESULT TriggerPin(GUID PinCategory);
HRESULT StartStreamCapture();
HRESULT StopStreamCapture();

void ResizeVideoWindow()
{
    // Resize the video preview window to match owner window size
    if(g_videoWindow!=NULL)
    {
        RECT rc;
        // Make the preview video fill our window
        GetClientRect(ghApp, &rc);
        rc.top+=TB_Height;
        g_videoWindow->SetWindowPosition(rc.left, rc.top, rc.right-rc.left, rc.bottom-rc.top);
        //CommandBar_DrawMenuBar(ghCB, 0);
    }
}

HRESULT SetupVideoWindow()
{
    HRESULT hr = ERROR_SUCCESS;

    if(g_videoWindow!=NULL)
    {
        //Set the video window to be a child of the main window
        hr = g_videoWindow->put_Owner((OAHWND)ghApp);
        if (FAILED(hr))
            return hr;

        // Set video window style
        hr = g_videoWindow->put_WindowStyle(WS_CHILD | WS_CLIPCHILDREN);
        if (FAILED(hr))
            return hr;

        // Use helper function to position video window in client rect 
        // of main application window
        ResizeVideoWindow();

        // Make the video window visible, now that it is properly positioned
        hr = g_videoWindow->put_Visible(OATRUE);
        if (FAILED(hr))
            return hr;
        hr = m_pEventEx->SetNotifyWindow((OAHWND)ghApp, WM_GRAPHNOTIFY, NULL);
        if (FAILED(hr))
            return hr;
    }
    return hr;
}



/*void StopCapturing()
{
RECT rect;
//CloseInterfaces();

// Redraw menu bar since we've probably resized the main window
CommandBar_DrawMenuBar(ghCB, 0);
GetClientRect(ghApp, &rect);
InvalidateRect(ghApp, &rect, TRUE);

}*/

LRESULT CALLBACK WndMainProc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
        case WM_CREATE:
            SetForegroundWindow(hwnd); // Ww have to do this otherwise this dialog will open in background the second time!
            return 0;

    case WM_ERASEBKGND:
        break;

    case WM_GRAPHNOTIFY:
        HandleGraphEvent();
        break;

    case WM_MOVE:
    case WM_SIZE:
        ResizeVideoWindow();
        if (g_videoWindow) 
            g_videoWindow->NotifyOwnerMessage((LONG_PTR) hwnd, message, wParam, lParam);
        return 0;

    case WM_DESTROY:
        PostQuitMessage(0);
        return 0;

    case WM_KEYDOWN:
        switch(wParam) 
        {
        case 'c':
        case 'C':
            {
                BOOL bCapturing=SendMessage(s_hWndToolbar, TB_ISBUTTONCHECKED, (WPARAM)ID_TB_CAPTURE, 0);
                if(bCapturing)
                    StopStreamCapture();
                else
                    StartStreamCapture();

                SendMessage(s_hWndToolbar, TB_CHECKBUTTON, (WPARAM)ID_TB_CAPTURE, MAKELONG(!bCapturing,0));

                break;
            }

        case 's':
        case 'S':
            TriggerPin(PIN_CATEGORY_STILL);
            break;
        }
        break;

    case WM_COMMAND:	
        {
            switch(wParam)
            { // Menus

            case ID_TB_GRAPH:
                {
                    if(SendMessage(s_hWndToolbar, TB_ISBUTTONCHECKED, (WPARAM)ID_TB_GRAPH, 0))
                        m_pControl->Run();
                    else
                        m_pControl->Stop();

                    /*{
                        TBBUTTONINFO tbbi;
                        tbbi.cbSize=sizeof(TBBUTTONINFO);
                        tbbi.pszText = bRunning ? L"Run Graph" : "Stop Graph";
                        tbbi.dwMask = TBIF_TEXT;
                        SendMessage(s_hWndToolbar, TB_SETBUTTONINFO, (WPARAM)ID_FILE_EXIT, (LPARAM)MAKELONG(FALSE,0));
                    }*/

                break;
                }

            case ID_TB_CAPTURE:
                if(SendMessage(s_hWndToolbar, TB_ISBUTTONCHECKED, (WPARAM)ID_TB_CAPTURE, 0))
                    StartStreamCapture();
                else
                    StopStreamCapture();

                //SendMessage(s_hWndToolbar, TB_HIDEBUTTON, (WPARAM)ID_VIDEO, (LPARAM)MAKELONG(FALSE,0));
                //SendMessage(s_hWndToolbar, TB_HIDEBUTTON, (WPARAM)ID_FILE_STOP, (LPARAM)MAKELONG(TRUE,0));
                break;

            case ID_TB_SNAP:
                TriggerPin(PIN_CATEGORY_STILL);
                break;
            case ID_TB_CONFIG:
                //GetSensorMode(L"CaptureMode", &g_CaptureWidth, &g_CaptureHeight, &g_CaptureFPS, FALSE);
                break;

            } // Menus
        }
        break;
    }

    //Pass this message to the video window for notification of system changes
    //if (g_videoWindow) g_videoWindow->NotifyOwnerMessage((LONG_PTR) hwnd, message, wParam, lParam);
    return DefWindowProc (hwnd , message, wParam, lParam);
}

HWND ToolBar_Create(HWND hWndParent) {
    // Declare and initialize local constants.
    const int ImageListID    = 0;
    const DWORD buttonStyles = 0; //TBSTYLE_AUTOSIZE;

    // Create the toolbar.
    s_hWndToolbar = CreateWindowEx(0, TOOLBARCLASSNAME, NULL, 
        WS_CHILD | TBSTYLE_WRAPABLE, 0, 0, 0, 0, 
        hWndParent, NULL, NULL, NULL);

    if (s_hWndToolbar == NULL)
        return NULL;

    g_hImageList = ImageList_LoadImage(NULL, MAKEINTRESOURCE(IDR_TOOLBAR1), 16, 0, CLR_NONE, IMAGE_BITMAP, 0);

    // Set the image list.
    SendMessage(s_hWndToolbar, TB_SETIMAGELIST, 
        (WPARAM)ImageListID, 
        (LPARAM)g_hImageList);


    TBBUTTON tbButtons[] = 
    {
        { 0,	ID_TB_GRAPH,	TBSTATE_ENABLED|TBSTATE_CHECKED, TBSTYLE_CHECK, {0}, 0, (INT_PTR)L"Graph"},
        { 0,	0,				TBSTATE_ENABLED, TBSTYLE_SEP,  {0}, 0},
        { 1,	ID_TB_CAPTURE,	TBSTATE_ENABLED, TBSTYLE_CHECK, {0}, 0, (INT_PTR)L"Capture" },
        { 2,	ID_TB_SNAP,		TBSTATE_ENABLED, TBSTYLE_BUTTON, {0}, 0, (INT_PTR)L"Still"},
        { 0,	0,				TBSTATE_ENABLED, TBSTYLE_SEP,  {0}, 0},
        { 4,	ID_TB_CONFIG,	TBSTATE_ENABLED, TBSTYLE_BUTTON,  {0}, 0, (INT_PTR)L"Config"},
    };

    // Add buttons.
    SendMessage(s_hWndToolbar, TB_BUTTONSTRUCTSIZE, (WPARAM)sizeof(TBBUTTON), 0);
    SendMessage(s_hWndToolbar, TB_ADDBUTTONS, (WPARAM)sizeof(tbButtons)/sizeof(TBBUTTON), (LPARAM)&tbButtons);

    // Resize the toolbar, and then show it.
    SendMessage(s_hWndToolbar, TB_AUTOSIZE, 0, 0);
    {
        RECT r;
        GetWindowRect(s_hWndToolbar, &r);
        TB_Height = r.bottom-r.top;
    }
    ShowWindow(s_hWndToolbar,  TRUE);

    return s_hWndToolbar;
}

HWND CreateVideoWindow()
{
    HINSTANCE hInstance = NULL;
    WNDCLASS wc;	
    //HRESULT hr;	

    //Create Instance of COM to be used for this Test.
    /*hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);
    if (FAILED(hr))
    {
    printf("CoInitializeEx failed.\n");
    goto CleanUp;
    }*/

    // Register the window class
    ZeroMemory(&wc, sizeof wc);
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc   = WndMainProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance     = hInstance;
    wc.lpszClassName = TEXT("Capture Preview");
    wc.lpszMenuName  = NULL;
    wc.hbrBackground = (HBRUSH)GetStockObject(GRAY_BRUSH);
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wc.hIcon         = NULL;
    if(!RegisterClass(&wc))
    {
        return 0;
    }

    // Create the main window.  The WS_CLIPCHILDREN style is required.
    ghApp = CreateWindowW(TEXT("Capture Preview"), TEXT("Video Capture Previewer"),
        WS_CAPTION | WS_CLIPCHILDREN  |WS_SIZEBOX |WS_OVERLAPPED|WS_MAXIMIZEBOX|WS_SYSMENU|WS_VISIBLE,
        //WS_OVERLAPPED | WS_CAPTION | WS_THICKFRAME | WS_SYSMENU,
        CW_USEDEFAULT, CW_USEDEFAULT,//534, 72+24,
        640,480-24,
        0, 0, hInstance, 0);

    if(ghApp)
    {
        //// doesn't work....
        //// Add menu bar
        //ghCB = CommandBar_Create(hInstance, ghApp, IDR_CMDBAR);
        //if(ghCB)
        //{
        //	CommandBar_InsertMenubar(ghCB, hInstance, IDR_MENU, 0);			
        //	//CommandBar_AddAdornments(ghCB, 0, 0);
        //	CommandBar_DrawMenuBar(ghCB, 0);
        //}
        //else { 
        //	err = GetLastError();
        //	printf("Failed creating CommandBar: Error %i.\n", err);
        //}
        ToolBar_Create(ghApp);
        return ghApp;
    }

    //CleanUp:
    return 0;
}



#define IBF_EDIT    0x100
#define IBF_LIST    0x200

typedef struct INPUTBOXPARAMS {
    HINSTANCE hInstance;
    LPTSTR lpDialogText;
    LPTSTR lpLabelText;
    LPTSTR lpComboInit;
    DWORD flags;
    TCHAR retText[1024];
    DWORD retVal;
} INPUTBOXPARAMS;

#define IBIDC_COMBO    100
#define IBIDC_CAPTION    101

#define H_MARGIN    8
#define V_MARGIN    8
//#define TEXT_H    64
#define COMBO_H    20
#define BUTTON_W    64
#define BUTTON_H    24

LRESULT CALLBACK InputDialogProc (
                                HWND hwndDlg,   // Handle to the dialog box.
                                UINT uMsg,      // Message.
                                WPARAM wParam,  // First message parameter.
                                LPARAM lParam)  // Second message parameter.
{
    static DWORD TEXT_H=0;

    switch(uMsg)
    {
        case WM_CREATE:
        {
            DWORD dwType=0;
            LPCREATESTRUCT lpcs = (LPCREATESTRUCT) lParam;
            INPUTBOXPARAMS* pIbp = (INPUTBOXPARAMS*)lpcs->lpCreateParams;

            SetWindowLong(hwndDlg, GWL_USERDATA, (LONG)pIbp); // Save InputBox Instance Data

            //Compute Required WindowSize
            {
                RECT caption_rect={0};
                HDC hDC = GetDC(hwndDlg);
                caption_rect.right=GetSystemMetrics(SM_CXSCREEN)*5/8; // Allow maximum 5/8 of ScreenSize
                DrawText(hDC, pIbp->lpLabelText, -1, &caption_rect, DT_CALCRECT|DT_WORDBREAK);
                ReleaseDC(hwndDlg, hDC);
                TEXT_H = caption_rect.bottom;
                {
                    DWORD w,h, nc_width, nc_height;
                    RECT rcClient;
                    GetClientRect(hwndDlg, &rcClient);
                    nc_width = 100 - rcClient.right;
                    nc_height = 100 - rcClient.bottom;
                    w = max(caption_rect.right+H_MARGIN*2+2, BUTTON_W*5/2);
                    h = caption_rect.bottom+V_MARGIN*4+BUTTON_H+COMBO_H;
                    SetWindowPos(hwndDlg, NULL, (GetSystemMetrics(SM_CXSCREEN)-(w+nc_width))/2, (GetSystemMetrics(SM_CYSCREEN)-(h+nc_height))/2, w+nc_width, h+nc_height, 0);
                }
            }

            // Create All Controls and position/size them
        {
            RECT rcClient;
            GetClientRect(hwndDlg, &rcClient);
            CreateWindowW(TEXT("STATIC"), pIbp->lpLabelText,
                0 | WS_VISIBLE | WS_CHILD,
                H_MARGIN, V_MARGIN, rcClient.right-H_MARGIN*2, TEXT_H, hwndDlg, (HMENU)IBIDC_CAPTION, NULL,
                NULL);

            if(pIbp->flags & IBF_LIST)
                dwType = (pIbp->flags & IBF_LIST) ? CBS_DROPDOWNLIST : CBS_DROPDOWN;
            CreateWindowW((pIbp->flags & IBF_LIST) ? TEXT("COMBOBOX") : TEXT("EDIT"), TEXT(""),
                dwType | CBS_OEMCONVERT | CBS_SORT | WS_VSCROLL | WS_TABSTOP | WS_VISIBLE | WS_CHILD,
                H_MARGIN, V_MARGIN + TEXT_H + V_MARGIN, rcClient.right-H_MARGIN*2, 128, hwndDlg, (HMENU)IBIDC_COMBO, NULL,
                NULL);

            CreateWindowW(TEXT("BUTTON"), TEXT("OK"),
                BS_DEFPUSHBUTTON | WS_TABSTOP | WS_VISIBLE | WS_CHILD,
                H_MARGIN, rcClient.bottom - V_MARGIN - BUTTON_H,
                BUTTON_W, BUTTON_H,
                hwndDlg, (HMENU)IDOK, NULL,
                NULL);
            CreateWindowW(TEXT("BUTTON"), TEXT("Cancel"),
                BS_PUSHBUTTON | WS_TABSTOP | WS_VISIBLE | WS_CHILD,
                rcClient.right - BUTTON_W - H_MARGIN, rcClient.bottom - V_MARGIN - BUTTON_H,
                BUTTON_W, BUTTON_H,
                hwndDlg, (HMENU)IDCANCEL, NULL,
                NULL);
        }

        {
            LPTSTR pEntry = pIbp->lpComboInit; // ComboInit is a null separated stringlinst (double-null terminated)
            while(*pEntry)
            {            
                SendMessage(GetDlgItem(hwndDlg, IBIDC_COMBO), CB_ADDSTRING, 0, (LPARAM)pEntry);
                pEntry+=lstrlen(pEntry)+1; // Next entry
            }
            SendMessage(GetDlgItem(hwndDlg, IBIDC_COMBO), CB_SETCURSEL, 0, 0);
            //SendMessage(GetDlgItem(hwndDlg, IBIDC_COMBO), CB_SHOWDROPDOWN, TRUE, 0);
        }

        SetForegroundWindow(hwndDlg); // Ww have to do this otherwise this dialog will open in background the second time!
        SetFocus(GetDlgItem(hwndDlg, IBIDC_COMBO));

        return TRUE;
        }
    
    case WM_SIZE:
        {
            INPUTBOXPARAMS* pIbp = (INPUTBOXPARAMS*)GetWindowLong(hwndDlg, GWL_USERDATA);

            RECT rcClient;
            GetClientRect(hwndDlg, &rcClient);
            // Recompute Needed Text Height based on current window Size
            {
                RECT caption_rect={0};
                HDC hDC = GetDC(hwndDlg);
                caption_rect.right = rcClient.right - H_MARGIN*2 - 2;
                DrawText(hDC, pIbp->lpLabelText, -1, &caption_rect, DT_CALCRECT|DT_WORDBREAK);
                ReleaseDC(hwndDlg, hDC);
                TEXT_H = caption_rect.bottom;
            }
            SetWindowPos(GetDlgItem(hwndDlg, IBIDC_CAPTION), NULL, H_MARGIN, V_MARGIN, rcClient.right-H_MARGIN*2, TEXT_H, SWP_SHOWWINDOW);
            SetWindowPos(GetDlgItem(hwndDlg, IBIDC_COMBO), NULL, H_MARGIN, V_MARGIN + TEXT_H + V_MARGIN , rcClient.right-H_MARGIN*2, 128, SWP_SHOWWINDOW);
            SetWindowPos(GetDlgItem(hwndDlg, IDOK), NULL, H_MARGIN, rcClient.bottom - V_MARGIN - BUTTON_H, BUTTON_W, BUTTON_H, SWP_SHOWWINDOW);
            SetWindowPos(GetDlgItem(hwndDlg, IDCANCEL), NULL, rcClient.right - BUTTON_W - H_MARGIN, rcClient.bottom - V_MARGIN - BUTTON_H, BUTTON_W, BUTTON_H, SWP_SHOWWINDOW);
        }

        return TRUE;

    case WM_COMMAND:
        switch (LOWORD(wParam))
        {
        case 0: // default button
            wParam = (HIWORD(wParam)<<16) | IDOK;
        case IDOK:
        case IDCANCEL:
            {
                INPUTBOXPARAMS* pIbp = (INPUTBOXPARAMS*)GetWindowLong(hwndDlg, GWL_USERDATA);
                GetDlgItemText(hwndDlg, IBIDC_COMBO, pIbp->retText, 1024);
                pIbp->retVal=LOWORD(wParam);
                DestroyWindow(hwndDlg);
                break;
            }
        }
        break;
    case WM_DESTROY:
        PostQuitMessage(0);
        break;
    }
    
    return DefWindowProc (hwndDlg , uMsg, wParam, lParam);
}

#ifndef WS_EX_CONTROLPARENT
#define WS_EX_CONTROLPARENT     0x00010000L
#endif

DWORD InputBox(INPUTBOXPARAMS* pIbp)
{
    WNDCLASS wc;
    MSG msg={0};

    ZeroMemory(&wc, sizeof wc);
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc   = InputDialogProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 32;
    wc.hInstance     = pIbp->hInstance;
    wc.lpszClassName = TEXT("InputBoxDlg");
    wc.lpszMenuName  = NULL;
    wc.hbrBackground = ( HBRUSH )( COLOR_STATIC + 1 );//(HBRUSH)GetStockObject(LTGRAY_BRUSH);
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wc.hIcon         = NULL;
    if(!RegisterClass(&wc))
    {
        return 0;
    }

    // Create InputBox Window
    ghApp = CreateWindowEx(WS_EX_CONTROLPARENT, TEXT("InputBoxDlg"), pIbp->lpDialogText,
        WS_CAPTION | WS_CLIPCHILDREN |WS_SIZEBOX |WS_OVERLAPPED|WS_MAXIMIZEBOX|WS_SYSMENU|WS_VISIBLE,
        //WS_OVERLAPPED | WS_CAPTION | WS_THICKFRAME | WS_SYSMENU,
        CW_USEDEFAULT, CW_USEDEFAULT,//534, 72+24,
        100,100,
        0, 0, pIbp->hInstance, pIbp);

        // Main message loop
        while(GetMessage(&msg,NULL,0,0))
        {
            if(!IsDialogMessage(ghApp,&msg))
            {
                TranslateMessage(&msg);
                DispatchMessage(&msg);
            }
        }

    UnregisterClass(TEXT("InputBoxDlg"), pIbp->hInstance);
    return pIbp->retVal;
}


typedef struct SensorHalRec
{
    char* pName;
    pfnImagerGetHal pfnGetHal;
} SENSOR_HAL;

typedef SENSOR_HAL* (*PFN_IMAGERGETLIST)(void);
typedef NvBool (*PFN_IMAGERGETHAL)(NvOdmImagerHandle hImager);
//
void SelectSensor(BOOL bForceDialog)
{
    HKEY hKey = NULL;
    DWORD dwDisp;
    TCHAR szSensorName[64];
    DWORD dwDataSize = sizeof(szSensorName);

    szSensorName[0]=0;

    // Check for SensorName in Registry
    if (ERROR_SUCCESS != RegCreateKeyEx(HKEY_LOCAL_MACHINE, L"SOFTWARE\\NVIDIA Corporation\\NVCAM", 0, NULL, 0, 0, NULL, &hKey, &dwDisp))
        return;
    
    if(!bForceDialog)
        RegQueryValueEx(hKey, L"SensorName", NULL, NULL, (LPBYTE)&szSensorName, &dwDataSize);

    // Query Sensor List from CameraPlugin.dll and check for SensorName
    {
        BOOL bFoundSensor=FALSE;
        TCHAR szList[256];
        TCHAR* pList=szList;
        DWORD len=0;
        HINSTANCE hMod = LoadLibrary(L"CameraPlugin.dll");
        PFN_IMAGERGETLIST pImagerGetList = (PFN_IMAGERGETLIST)GetProcAddress(hMod, L"SensorPlugin_GetList");
        SENSOR_HAL* pSensorList = pImagerGetList();
        // Build a list of all Sensors. if we find one named SensorName we abort and exit
        while(pSensorList->pName)
        {
            len=swprintf_s(pList, sizeof(szList), L"%S", pSensorList->pName);
            if(szSensorName[0] && wcsstr(pList, szSensorName)) {bFoundSensor=TRUE; break;}
            pList+=len+1; // Go past the NULL termination (to leave it in on next iteration, so we have a NULL separated list)
            pSensorList++;
        }
        *pList=0; // Add second NULL termination to end the list

        // If SensorName is not found in list, show a dialog to let the user select one from list
        if(!bFoundSensor)
        {
            INPUTBOXPARAMS ibp;
            ibp.flags = IBF_LIST;
            ibp.lpDialogText = L"Select Camera Sensor";
            ibp.lpLabelText = L"Select a Camera Sensor from the list.\nThis will we saved to the registry for later.\n(HKLM\\SOFTWARE\\NVIDIA Corporation\\NVCAM\\SensorName)";

            ibp.lpComboInit = szList;
            InputBox(&ibp);

            if(ibp.retVal==IDOK)
                RegSetValueEx(hKey, L"SensorName", 0, REG_SZ, (LPBYTE)ibp.retText, (lstrlen(ibp.retText)+1)*sizeof(TCHAR));
        }
        FreeLibrary(hMod);
    }
    RegCloseKey(hKey);
}

// pszPinNameMode can be "PreviewMode", "CaptureMode", "StillMode"
void GetSensorMode(TCHAR* pszPinNameMode, DWORD* pWidth, DWORD* pHeight, DWORD* pFps, BOOL bForceDialog)
{
    HKEY hKey = NULL;
    DWORD dwDisp;

    if (ERROR_SUCCESS == RegCreateKeyEx(HKEY_LOCAL_MACHINE, L"SOFTWARE\\NVIDIA Corporation\\NVCAM", 0, NULL, 0, 0, NULL, &hKey, &dwDisp))
    {
        TCHAR szModeStr[64];
        DWORD dwDataSize = sizeof(szModeStr);
        if(!bForceDialog && ERROR_SUCCESS == RegQueryValueEx(hKey, pszPinNameMode, NULL, NULL, (LPBYTE)&szModeStr, &dwDataSize))
        {
            swscanf(szModeStr, L"%dx%dx%d", pWidth, pHeight, pFps);
        }
        else
        {
            NvOdmImagerSensorMode* pSensorModes;
            NvS32 i, numModes;
            HINSTANCE hMod = LoadLibrary(L"CameraPlugin.dll");
            PFN_IMAGERGETHAL pGetHal = (PFN_IMAGERGETHAL)GetProcAddress(hMod, L"SensorPlugin_GetHal");
            struct NvOdmImagerRec Imager;
            NvOdmImagerSensor sensor;
            NvOdmImagerHandle hImager = &Imager;
            hImager->pSensor = &sensor;

            pGetHal(hImager);
            hImager->pSensor->pfnOpen(hImager);
            numModes = -1;
            hImager->pSensor->pfnListModes(hImager, NULL, &numModes);
            pSensorModes = (NvOdmImagerSensorMode*)malloc(numModes*sizeof(NvOdmImagerSensorMode));
            numModes = -1;
            hImager->pSensor->pfnListModes(hImager, pSensorModes, &numModes);
            hImager->pSensor->pfnClose(hImager);
            FreeLibrary(hMod);

            {
                TCHAR szList[256];
                TCHAR* pList=szList;
                DWORD len=0;
                for(i=0; i<numModes; i++)
                {
                    len=swprintf_s(pList, sizeof(szList), L"%dx%dx%d", pSensorModes[i].ActiveDimensions.width, pSensorModes[i].ActiveDimensions.height, (DWORD)pSensorModes[i].PeakFrameRate);
                    pList+=len+1; // Go past the NULL termination (to leave it in on next iteration, so we have a NULL separated list)
                }
                *pList=0; // Add second NULL termination to end the list

                {
                    INPUTBOXPARAMS ibp;
                    ibp.flags = IBF_LIST;
                    ibp.lpDialogText = L"Select a Mode";
                    ibp.lpLabelText = L"Select a Camera Mode from the list.\nThis will we saved to the registry for later.\n(HKLM\\SOFTWARE\\NVIDIA Corporation\\NVCAM\\SensorName)";

                    ibp.lpComboInit = szList;
                    InputBox(&ibp);

                    if(ibp.retVal==IDOK)
                    {
                        DWORD SensorMode=-1, i=0;
                        pList=szList;

                        while(*pList)
                        {
                            if(lstrcmp(ibp.retText, pList)==0)
                            {
                                SensorMode=i;
                                break;
                            }
                            pList+=lstrlen(pList)+1; // Next entry
                            i++;
                        }
                        RegSetValueEx(hKey, pszPinNameMode, 0, REG_SZ, (LPBYTE)ibp.retText, (lstrlen(ibp.retText)+1)*sizeof(TCHAR));
                        RegSetValueEx(hKey, L"SensorMode", 0, REG_DWORD, (LPBYTE)&SensorMode, sizeof(DWORD));
                        swscanf(ibp.retText, L"%dx%dx%d", pWidth, pHeight, pFps);
                    }
                }
                
            }
            
        }
    }

}
