/*
 * Copyright (c) 2008 NVIDIA Corporation.  All Rights Reserved.
 *
 * NVIDIA Corporation and its licensors retain all intellectual property and
 * proprietary rights in and to this software and related documentation.  Any
 * use, reproduction, disclosure or distribution of this software and related
 * documentation without an express license agreement from NVIDIA Corporation
 * is strictly prohibited.
 */

#ifndef INCLUDED_ENCODE_TEST_DEFINES_H
#define INCLUDED_ENCODE_TEST_DEFINES_H

/**
 * NVIDIA Video Renderer Filter is a video renderer filter
 * responsible for rendering video for DirectSow playback and capture
 * graphs.
 *
 * @par GUID:
 *    {4362FF77-2F31-498c-B7E3-0FB18BE5EE26}
 *
 * @par Supports:
 * Pluggabiltiy with 3rd-party filters for RGB565 and YV12 formats.
 *
 * @par Media Types and Subtypes exposed by NvVideoRenderer
 * (to connect with 3rd Party Filters)
 *
 * Input Stream
 *  NVIDIA Specific types exposed by NvVideoRenderer Filter (to connect with NV Filters):
 * - MEDIATYPE_NvVideo (Major Type) and MEDIASUBTYPE_NvYUV420 (Minor Type)
 *
 * Standard Major and Minor types are exposed by NvVideoRenderer Filter
 * (to connect with 3rd Party Filters):
 * - MEDIATYPE_Video (Major Type) and MEDIASUBTYPE_RGB565 (Minor Type MEDIASUBTYPE_RGB565)
 * - MEDIATYPE_Video (Major Type) and MEDIASUBTYPE_YV12 (Minor Type MEDIASUBTYPE_YV12)
 * - MEDIATYPE_Video (Major Type) and MEDIASUBTYPE_I420 (Minor Type MEDIASUBTYPE_I420)
 */
// {4362FF77-2F31-498c-B7E3-0FB18BE5EE26}
DEFINE_GUID(CLSID_NvVideoRenderer,
0x4362FF77, 0x2F31, 0x498C, 0xB7, 0xE3, 0xF, 0xB1, 0x8B, 0xE5, 0xEE, 0x26);


/**
 * NVIDIA Mux File Writer Filter
 *
 * @par Supports:
 * - MUX of encoded H.264, H.263, MPEG-4 video data
 * - Audio AAC-LC, AAC+, E-AAC+, AMR-NB, AMR-WB audio data
 * - File writing into 3GP file format
 * - Writing JPEG stream to file
 * @note Still image dimension cannot be greater than the capture dimension
 *   when creating both still image and capture pin at the same time.
 *
 * @par GUID:
 * {74f08d23-78e3-4f2a-bd84-532038d9cede}
 *
 */
DEFINE_GUID(CLSID_Nv3GP2MuxFilter,
0x74f08d23, 0x78e3, 0x4f2a, 0xbd, 0x84, 0x53, 0x20, 0x38, 0xd9, 0xce, 0xde);


/**
 * NVIDIA Video Encoder Filter (DirectShow)
 *
 * @par Supports:
 * H.264, H.263, and MPEG-4 video encoding formats
 *
 * @par GUID:
 * {E5D912CD-FE4D-4ee2-A9E1-C3B7984029B0}
 *
 * The following list shows the output media subtype (minor type) supported
 * by thier respective media type (major type).
 *
 * @note For all video/still-image output types, the major types
 *   \c MEDIATYPE_NvVideo as well as \c MEDIATYPE_Video are supported.
 *
 *  - MEDIASUBTYPE_H264Video (minor type), MEDIATYPE_NvVideo (major type)
 *  - MEDIASUBTYPE_MPEG4Video (minor type), MEDIATYPE_NvVideo (major type)
 *  - MEDIASUBTYPE_H263Video (minor type), MEDIATYPE_NvVideo (major type)
 *  - MEDIASUBTYPE_NvJPEG (minor type), MEDIATYPE_NvVideo (major type)
 *  - MEDIASUBTYPE_H264Video (minor type), MEDIATYPE_Video (major type)
 *  - MEDIASUBTYPE_MPEG4Video (minor type), MEDIATYPE_Video (major type)
 *  - MEDIASUBTYPE_H263Video (minor type), MEDIATYPE_Video (major type)
 *  - MEDIASUBTYPE_IJPG (minor type), MEDIATYPE_Video (major type)
 *
 * The video encoder's frame dimension is the same as that of the frame
 * dimension set on the video capture filter. The frame length (sample size)
 * is decided based upon frame dimension and bit-count at connection time.
 *
 * The following list shows the input type supported (minor type) supported by
 * their respective media type (major type):
 *
 *  - MEDIASUBTYPE_NvMMBuffer (minor type), MEDIATYPE_NvVideo (major type)
 *  - MEDIASUBTYPE_YV12 (minor type), MEDIATYPE_VideoBuffered (major type)
 *  - MEDIASUBTYPE_RGB565 (minor type), MEDIATYPE_VideoBuffered (major type)
 */
DEFINE_GUID(CLSID_NvVideoEncoder,0xe5d912cd, 0xfe4d, 0x4ee2, 0xa9, 0xe1, 0xc3, 0xb7, 0x98, 0x40, 0x29, 0xb0);

/**
 * NVIDIA Video Capture Filter
 *
 * The video capture filter is a 3-pin filter. This filter exposes the
 * preview pin, capture pin, and still pin.
 *
 * Supported media types (major and minor types) on the preview pin:
 * - MEDIASUBTYPE_NvYUV420 (minor type), MEDIATYPE_NvRendererVideo (major type)
 * - MEDIASUBTYPE_RGB565 (minor type), MEDIATYPE_Video (major type)
 * - MEDIASUBTYPE_YV12 (minor type), MEDIATYPE_Video (major type)
 *
 * Supported media types (major and minor types) on the capture pin:
 * - MEDIASUBTYPE_NvMMBuffer (minor type), MEDIATYPE_NvVideo (major type)
 * - MEDIASUBTYPE_YV12 (minor type), MEDIATYPE_VideoBuffered (major type)
 * - MEDIASUBTYPE_RGB565 (minor type), MEDIATYPE_VideoBuffered (major type)
 *
 * Supported media type (major and minor type) on the still pin:
 * - MEDIASUBTYPE_NvMMBuffer (minor type), MEDIATYPE_NvVideo (major type)
 * - MEDIASUBTYPE_YV12 (minor type), MEDIATYPE_VideoBuffered (major type)
 * - MEDIASUBTYPE_RGB565 (minor type), MEDIATYPE_VideoBuffered (major type)
 * - MEDIASUBTYPE_Bayer (minor type), MEDIATYPE_NvVideo (major type)
 *
 * @par Setting Frame Dimension
 *
 * The frame dimension can be set for all 3 pins separately using the
 * <b>IAMStreamConfig::SetFormat</b> method. The frame dimension on a pin must
 * be set before that pin is rendered (connected to another filter). If
 * not set, the capture filter will use a default frame dimension.
 *
 * If all 3 pins, preview, capture and still are connected at the same time
 * then still pin's dimension cannot be more than the capture pin's dimension.
 * This limitation does not apply when either the capture pin or the still
 * pin remains unconnected.
 *
 * @code
 *
 * VIDEOINFOHEADER vih = {0};
 * vih.AvgTimePerFrame = UNITS / 30;        // Frame rate
 * vih.bmiHeader.biWidth = 720;             // Frame width
 * vih.bmiHeader.biHeight = 480;            // Frame height
 *
 * AM_MEDIA_TYPE amt = {0};
 * amt.formattype = FORMAT_VideoInfo;
 * amt.pbFormat = (BYTE *)&vih;
 * amt.cbFormat = sizeof(VIDEOINFOHEADER);
 *
 * IAMStreamConfig *pStreamConfig = NULL;
 *
 * // Code to enumerate pins, determine pin category and QueryInterface for
 * // IAMStreamConfig from pin's IPin interface.
 *
 * pStreamConfig->SetFormat(&amt);
 *
 * @endcode
 *
 * @par See also
 * For more information, see MSDN documentation on @b IAMStreamConfig at:
 * <a href="http://msdn.microsoft.com/en-us/library/aa915620.aspx" 
 * target="_blank">http://msdn.microsoft.com/en-us/library/aa915620.aspx</a>
 * 
 * @par GUID:
 * {cdc2b6ec-87c0-489d-be56-f522d8406c4b}
 */
DEFINE_GUID(CLSID_NvVideoCapture,
0xCDC2B6EC, 0x87C0, 0x489D, 0xBE, 0x56, 0xF5, 0x22, 0xD8, 0x40, 0x6C, 0x4B);

/**
 * NVIDIA Audio Capture Filter
 *
 * The audio capture filter exposes 1 output pin. Supported major type and
 * minor type:
 * - MEDIASUBTYPE_PCM (minor type), MEDIATYPE_AudioBuffered (major type)
 *
 * @par GUID:
 * {b8300c7c-3390-43ab-acc0-29e58e7e390e}
 */
DEFINE_GUID(CLSID_NvAudioCapture,
0xb8300c7c, 0x3390, 0x43ab, 0xac, 0xc0, 0x29, 0xe5, 0x8e, 0x7e, 0x39, 0xe);

/**
 * NVIDIA Audio Encoder Filter (DirectShow)
 *
 * @par Supports:
 * AAC-LC, AAC+, E-AAC+, AMR-NB, AMR-WB
 *
 * @par GUID:
 * {c98a8f9b-1074-4671-b5a9-58c8f42179c6}
 *
 * The following list shows the output media subtype (minor type) supported
 * by thier respective NVIDIA media type (major type).
 *
 * @note For all audio/speech codecs, the output type
 *   major type is \c MEDIATYPE_NvAudio.
 * 
 *  - MEDIASUBTYPE_AMRAudio (minor type), MEDIATYPE_NvAudio (major type)
 *  - MEDIASUBTYPE_AWBAudio (minor type), MEDIATYPE_NvAudio (major type)
 *  - MEDIASUBTYPE_AACAudio (minor type), MEDIATYPE_NvAudio (major type)
 *  - MEDIASUBTYPE_AACPLUSAudio (minor type), MEDIATYPE_NvAudio (major type)
 *  - MEDIASUBTYPE_EAACPLUSAudio (minor type), MEDIATYPE_NvAudio (major type)
 *
 * The following shows the input type supported (minor type) supported by
 * its NVIDIA media type (major type):
 * - MEDIASUBTYPE_PCM (minor type), MEDIATYPE_NvAudioBuffered (major type)
 */
DEFINE_GUID(CLSID_NvAudioEncoder,
0xc98a8f9b, 0x1074, 0x4671, 0xb5, 0xa9, 0x58, 0xc8, 0xf4, 0x21, 0x79, 0xc6);
//#endif

/**
 * Property Set GUID to set filter-level properties. This is used by
 * NVIDIA Video Capture Filter and NVIDIA Audio Capture Filter.
 *
 * @par GUID:
 * {A211D09C-73BC-4fc8-B613-03471A9C69BF}
 *
 * The @b IKsPropertySet interface is used to set properties on
 * NVIDIA Video Capture Filter and NVIDIA Audio Capture Filter.
 * This interface is exposed on on the two capture filters.
 * For more information, see MSDN at:
 *    <a href="http://msdn.microsoft.com/en-us/library/aa929907.aspx" 
 *       target="_blank">
 *       http://msdn.microsoft.com/en-us/library/aa929907.aspx</a>
 *
 * All properties can be set anytime after the video or audio capture filter
 * is created. Most properties can be changed at run-time.
 *
 * See ::NvPropertyId enumeration for all supported Property IDs.
 *
 * @par Example
 * The following example shows setting the filter-level property on
 * NVIDIA Video Capture Filter. A similar method is used to set filter-level
 * properties on NVIDIA Audio Capture Filter.
 *
 * @code
 *
 * // QueryInterface for IKsPropertySet from IBaseFilter interface or any
 * // interface exposed on NVIDIA Video Capture Filter. In this example, we
 * // will query IKsPropertySet from IBaseFilter interface of NVIDIA
 * // Video Capture Filter.
 * pCapBaseFilter->QueryInterface(IID_IKsProperty, (void **)pCapFilterPropSet);
 *
 * // Call IKsPropertySet::Set to set a filter-level property. In this example,
 * // we set zoom.
 * NvU16 Zoom = 200;
 * pCapPinPropSet->Set(
 *     PROPSET_NvFilter,                // Filter-level property.
 *     NvPropertyId_Zoom,               // Property identifier.
 *     NULL,
 *     0,
 *     &Zoom,                           // Property value.
 *     sizeof(NvU16));                  // Type of property value.
 *
 * @endcode
 */
DEFINE_GUID(PROPSET_NvFilter,
0xa211d09c, 0x73bc, 0x4fc8, 0xb6, 0x13, 0x3, 0x47, 0x1a, 0x9c, 0x69, 0xbf);


/*@}*/

/** @name Custom Events
 * These are custom graph events sent by capture and encode related filters
 * to the application. Filters post these events to the filter
 * graph using the @b IMediaEventSink interface. Applications can
 * trap these events using the @b IMediaEvent and @b IMediaEventEx
 * interfaces.
 *              
 * For more information, see MSDN documentation at:
 * - <a href="http://msdn.microsoft.com/en-us/library/ms783650(VS.85).aspx"
 *       target="_blank">http://msdn.microsoft.com/en-us/library/ms783650(VS.85).aspx</a> for DirectShow event notification
 * - <a href="http://msdn.microsoft.com/en-us/library/aa915935.aspx"
 *       target="_blank">http://msdn.microsoft.com/en-us/library/aa915935.aspx</a> for @b IMediaEvent
 * - <a href="http://msdn.microsoft.com/en-us/library/aa928602.aspx"
 *       target="_blank">http://msdn.microsoft.com/en-us/library/aa928602.aspx</a> for @b IMediaEventEx
 */
/*@{*/

/**
 * This is the same as \c EC_USER, which is defined in evcode.h
 * to be 0x80000. All custom event codes should begin from \c EC_USER. 
 */
enum { EC_NV_USER = 0x8000 };

/**
 * This is the base for all graph events sent by capture and encode
 * related filters. This is to avoid conflicts with graph events sent by
 * playback and decode related filters.
 */
enum { EC_NV_ENC = 0x1000 };

/**
 * This event is sent by NVIDIA Video Capture Filter when the
 * filter was able to achieve auto focus in response to half press.
 *
 * @par To receive this event:
 * -# Enable auto focus by setting \c AutoFocus to NV_TRUE using
 *    ::NvPropertyId_CameraProperties filter-level property.
 * -# Optionally set \c ExposureTime and \c WhiteBalance.
 * -# Enable half press by setting ::NvPropertyId_HalfPress
 *    filter-level property.
 */
enum { EC_NV_AUTO_FOCUS_ACHIEVED = EC_NV_USER + EC_NV_ENC + 0x1 };

/**
 * This event is sent by NVIDIA Video Capture Filter when the
 * filter was able to achieve auto exposure in response to half press.
 *
 * @par To receive this event:
 * -# Enable auto exposure by setting \c ExposureTime to \a Auto using
 *    ::NvPropertyId_CameraProperties filter-level property.
 * -# Optionally set \c AutoFocus and \c WhiteBalance.
 * -# Enable half press by setting ::NvPropertyId_HalfPress
 *    filter-level property.
 */
enum { EC_NV_AUTO_EXPOSURE_ACHIEVED = EC_NV_USER + EC_NV_ENC + 0x2 };

/**
 * This event is sent by NVIDIA Video Capture Filter when the
 * filter was able to achieve auto white balance in response to
 * half press.
 * 
 * @par To receive this event:
 * -# Enable auto white balance by setting \c WhiteBalance to \a Auto
 *    using ::NvPropertyId_CameraProperties filter-level property.
 * -# Optionally set \c AutoFocus and \c ExposureTime.
 * -# Enable half press by setting ::NvPropertyId_HalfPress
 *    filter-level property.
 */
enum { EC_NV_AUTO_WHITE_BALANCE_ACHIEVED = EC_NV_USER + EC_NV_ENC + 0x3 };

/**
 * This event is sent by NVIDIA Video Capture Filter when the
 * filter was unable to achieve auto focus in response to half
 * press in a certain amount of time.
 * This event is sent instead of ::EC_NV_AUTO_FOCUS_ACHIEVED if
 * auto focus times out.
 */
enum { EC_NV_AUTO_FOCUS_TIMED_OUT = EC_NV_USER + EC_NV_ENC + 0x4 };

/**
 * This event is sent by NVIDIA Video Capture Filter when the
 * filter was unable to achieve auto exposure in response to half
 * press in a certain aomount of time.
 * This event is sent instead of ::EC_NV_AUTO_EXPOSURE_ACHIEVED if
 * auto exposure times out.
 */
enum { EC_NV_AUTO_EXPOSURE_TIMED_OUT = EC_NV_USER + EC_NV_ENC + 0x5 };

/**
 * This event is sent by NVIDIA Video Capture Filter when the
 * filter was unable to achieve auto focus in response to half
 * press in a certain amount of time.
 * This event is sent instead of ::EC_NV_AUTO_FOCUS_ACHIEVED if
 * auto focus times out.
 */
enum { EC_NV_AUTO_WHITE_BALANCE_TIMED_OUT = EC_NV_USER + EC_NV_ENC + 0x6 };

/**
 * This event is sent by NVIDIA Video Capture Filter when the pause
 * still capture feature is enabled.
 * 
 * To enable the pause still capture feature, use the
 * ::NvPropertyId_PausePreviewAfterStillCapture property. After
 * a still capture is completed, NVIDIA Video Capture Filter
 * sends this event. The application can trap this event and
 * then pause the filter graph. During this time, the application
 * can show the captured image to the user for confirmation.
 * After confirmation, the application can then run the filter
 * graph again.
 * 
 */
enum { EC_NV_AUTO_PAUSE_STILL_CAPTURE_DONE = EC_NV_USER + EC_NV_ENC + 0x7 };

/**
 * This event is sent by NVIDIA Mux File Writer when it is done capturing
 * a still frame.
 *
 * An application can trigger NVIDIA's Video Capture Filter's still pin using
 * the <b>IAMVideoControl::SetMode</b> method. When triggered, the capture filter
 * delivers a still frame to the downstream filter. When NVIDIA Mux File Writer
 * filter is done with the still frame, it sends this event. The event is sent
 * once per trigger.
 *
 * For more information on <b>IAMVideoControl::SetMode</b>, see MSDN documentation at:
 * <a href="http://msdn.microsoft.com/en-us/library/aa915574.aspx"
 * target="_blank">http://msdn.microsoft.com/en-us/library/aa915574.aspx</a>
 */
enum { EC_NV_CAP_FILE_COMPLETED = EC_NV_USER + EC_NV_ENC + 0x8 };

/**
 * This event is sent by NVIDIA Mux File Writer for invalid user information 
 * or when User Data Size Exceeds 216 bytes. This is a warning message.
 * Mux Writer still continue to write data.
 */
enum { EC_NV_MUX_WRITER_INVALID_USER_DATA = EC_NV_USER + EC_NV_ENC + 0x9 };

/**
 * This event is sent by NVIDIA Mux File Writer when it has exhausted the 
 * 3GP File Size of 2 GB (Or File Size limit specified by user) as dictated 
 * by ISO MP4 version 0 which NVIDIA Mux File Writer supports. Upon receival
 * of this event Test can reinitialize the file name to continue dumping ]
 * the AV-Recorded data by setting property through ::NvPropertyId_MuxWriterReinitializeFilename.
 */
enum { EC_NV_MUX_WRITER_FILESIZE_LIMIT_EXCEED = EC_NV_USER + EC_NV_ENC + 0xA };

/**
 * This event is sent by NVIDIA Encoder Filter when it gets Unsupported resolutions 
 * property through ::EC_NV_RESOLUTION_NOT_SUPPORTED.
 */
enum { EC_NV_RESOLUTION_NOT_SUPPORTED = EC_NV_USER + EC_NV_ENC + 0xB };

/**
 * This event is sent by NVIDIA Video Capture Filter.
 * This will be used for non-fatal errors such as bad calibration data
 * or insufficient memory to use advanced features.
 * through ::EC_NV_VIDEO_CAPTURE_WARNING.
 * The type of warning will be send in param1 of Event Notify call.
 * Please refer NvMMBlockWarning in nvmm_event.h for all possible warning
 * associted with this notification.
 */
enum { EC_NV_VIDEO_CAPTURE_WARNING = EC_NV_USER + EC_NV_ENC + 0xC };

/**
 * This event is sent by NVIDIA Writer Filter.
 * This will be used to notify that BaseNumber of encoded file
 * has exceeded the value allowed by DCF i.e 9999
 * through ::EC_NV_MUX_WRITER_DCF_BASE_NUMBER_EXCEEDED.
 */
enum { EC_NV_MUX_WRITER_DCF_BASE_NUMBER_EXCEEDED = EC_NV_USER + EC_NV_ENC + 0xD };
/**
 * This event is sent by NVIDIA Video Capture Filter.
 * This will be used for SetaAttribute errors 
 * through ::EC_NV_SETATTRIBUTE_ERROR.
 */
enum { EC_NV_SETATTRIBUTE_ERROR = EC_NV_USER + EC_NV_ENC + 0xE };

/**
 * This event is sent by NVIDIA Video Capture Filter when the
 * filter is ready for another still trigger request.
 */
enum { EC_NV_STILL_CAPTURE_READY = EC_NV_USER + EC_NV_ENC + 0xF };

/**
 * This event is sent by NVIDIA Video Capture Filter when the
 * filter is ready for another video capture request.
 */
enum { EC_NV_VIDEO_CAPTURE_READY = EC_NV_USER + EC_NV_ENC + 0x10 };

#ifndef EC_CAP_FILE_WRITE_ERROR
#define EC_CAP_FILE_WRITE_ERROR                 0x104
#endif

/*@}*/

/**
 * Defines property IDs used to identify
 * properties of NVIDIA Video Capture Filter and NVIDIA Audio
 * Capture Filter. There are 2 types of Property ID sets:
 * - Filter-level property set identified by ::PROPSET_NvFilter, and
 * - Pin level property set identified by ::PROPSET_NvPin.
 *              
 * These properties are set using the IKsPropertySet interface
 * exposed by NVIDIA Video and NVIDIA Audio Capture Filter at pin and
 * filter level.
 */
typedef enum
{

    /**
     * Filter-level property to set camera properties.
     * Property value type is ::NvCameraProperties. Sets various camera
     * properties like auto focus, exposure time, white balance, etc.
     */
    NvPropertyId_CameraProperties = 4,

    /**
     * Filter-level property to set color properties.
     * Property value type is ::NvColorProperties. Sets various color
     * properties like hue, saturation, brightness, etc. 
     */
    NvPropertyId_ColorProperties,

    /**
     * Filter-level property to set zoom.
     * Property value type is NvU16. NVIDIA Video Capture Filter
     * supports zoom values from 100 to 800 (1.0-8.0) up to 8x digital
     * zoom.
     */
    NvPropertyId_Zoom,

    /**
     * Filter-level property to set smooth zoom time, in milliseconds.
     * Property value type is NvU32.
     */
    NvPropertyId_SmoothZoomTime,

    /**
     * Filter-level property to set smooth zoom abort. (Get is not
     * supported on this filter-level property.)
     * Property value type is NvBool.
     */
    NvPropertyId_SmoothZoomAbort,

    /**
     * Reserved.
     */
    NvPropertyId_ZoomMult,

    /**
     * Pin and filter-level property to set 2D transform.
     * Property value type is \c NvDdk2dTransform. This property is
     * supported at pin level as well as filter level. When set at the
     * filter level, NVIDIA Video Capture Filter will set the same
     * transform value on all of its pins.
     * @note Filter level does not support Get on this property. Pin
     * level supports both Get and Set.
     *
     * Supported transform values are:
     * - NvDdk2dTransform_None
     * - NvDdk2dTransform_Rotate90
     * - NvDdk2dTransform_Rotate180
     * - NvDdk2dTransform_Rotate270
     * - NvDdk2dTransform_FlipHorizontal (frame-flipping)
     * - NvDdk2dTransform_FlipVertical (frame-mirroring)
     * - NvDdk2dTransform_Transpose
     * - NvDdk2dTransform_InvTranspose
     *
     * The above values are defined in nvddk_2d.h
     */
    NvPropertyId_2dTransform,

    /**
     * Read-only filter-level property to get the current frame rate.
     * Property value type is NvF32. Used to get frame rate at which
     * NVIDIA Video Capture Filter is currently delivering frames.
     *
     * NVIDIA Video Capture Filter delivers frames between values
     * specified by ::NvPropertyId_FrameRateRange
     */
    NvPropertyId_CurrentFrameRate = 13,

    /**
     * Filter-level property to enable or disable image stabilization
     * Property value type is NvBool. Set to NV_TRUE to enable or
     * NV_FALSE to disable image stabilization.
     *
     * @note Turning this on implicitly disables video stabilization.
     */
    NvPropertyId_ImageStabilization,

    /**
     * Filter-level property to enable or disable video stabilization
     * Property value type is NvBool. Set to NV_TRUE to enable or
     * NV_FALSE to disable video stabilization.
     *
     * @note Turning this on implicitly disables image stabilization.
     */
    NvPropertyId_VideoStabilization,

    /**
     * Filter-level property to enable or disable image chroma filtering
     * Property value type is NvBool. Set to NV_TRUE to enable or
     * NV_FALSE to disable image chroma filtering.
     */
    NvPropertyId_ImageChromaFiltering,

    /**
     * Filter-level property to select active imager.
     * Property value type is NvU64. If the device has more
     * than 1 imagers, then this property is used to select between
     * them. The property values are numbers that identify the imager
     * on the device.
     *
     * @pre This property should be set before running the graph and should
     * be the first property to be set. If this property is set after setting
     * any other property that results in changing the currently selected
     * sensor, then the camera is reinitialized with the new sensor.
     * Reinitialization resets all properties to defaults. Hence, this property
     * must be set first.
     */
    NvPropertyId_ActiveImager,

    /**
     * Filter-level property to set and get the auto focus region.
     * The property value type is ::NvAutoFocusRegionNormalized. This property
     * can also be set at pin-level. When used at the pin level, the property
     * value type is ::NvAutoFocusRegion. Setting this property for one pin
     * will proportionately change auto focus region of all other pins since
     * there can be only auto focus region applied to all pins.
     *
     * Auto focus region will work only when auto focus is enabled. To enable
     * auto focus, set NvCameraProperties::AutoFocus to NV_TRUE and
     * set it using the ::NvPropertyId_CameraProperties
     * property ID.
     *
     * When specified at the filter level, the rectangle coordinates are
     * normalized to -1.0 to 1.0 on the X and Y axis. When specified at the
     * pin level, the rectangle coordinates are those of the actual frame.
     * As an example, if the video is 320 x 240 pixels and the application
     * wants to specify an auto focus region of 0, 0, 160, 120 (left half of
     * the video) then at filter level, the coordinates will be
     * -1.0, -1.0, 0.0, 0.0. On the other hand when specified at the pin level
     * then the coordinates will be 0, 0, 160, 120. This is so because at
     * pin level, the resolutions of preview, capture, and still can be
     * different. The pin level property is also specified for convenience.
     */
    NvPropertyId_AutoFocusRegion,

    /**
     * Filter-level and pin-level property to set cropping rectangle
     * Property value type is ::NvRect. Sets the cropping rectangle.
     * NVIDIA Video Capture Filter will crop all frames to this
     * rectangle before delivering them. When set at filter level, the
     * specified crop rectangle is applied to all 3 pins of the filter.
     * @note Filter level does not support Get on this property. Pin
     * level supports both Get and Set.
     */
    NvPropertyId_CropRect,

    /**
     * Filter-level property to set application type.
     * Property value type is ::NvAppType enumeration. NVIDIA Video
     * Capture filter manipulates time-stamps based on the application
     * type: Camcorder or Video Telephony (VT).
     *
     * @pre This property should be set before connection.
     */
    NvPropertyId_AppType,

    /**
     * Filter-level property to set edge enhancement (sharpness).
     * Property value type is ::NvEdgeEnhanceSettings structure.
     */
    NvPropertyId_EdgeEnhancement,

    /**
     * Filter-level property to repeat frames.
     * Property value type is ::NvRepeatedFrameSettings structure. When
     * enabled, NVIDIA Video Capture Filter will continuously deliver
     * the same frame until disabled. This property is also used to
     * continuously deliver an RGB565 or YUV420 (YV12) image provided
     * by the application.
     */
    NvPropertyId_RepeatFrame,

    /**
     * Filter-level property to enable and disable half press. (Get is not
     * supported on this filter-level property.)
     * Property value type is NvBool.
     * 
     * When set to NV_TRUE, NVIDIA Video Capture Filter will start
     * converging (achieving) auto focus, auto exposure, and/or auto
     * white balance properties. If the capture filter achieves one or
     * more of these properties then it will lock settings of those
     * properties. If the capture filter was unable to achieve one or
     * more of those properties in a certain time then it times out
     * for that property.
     * 
     * When set to NV_FALSE, NVIDIA Video Capture Filter will
     * unlock settings for auto focus, auto exposure, and/or auto
     * white balance properties.
     * 
     * NVIDIA Video Capture Filter notifies the application about
     * those properties that were achieved and those that timed out
     * using graph events. The following lists shows the different
     * events that are sent.
     * - Auto focus achieved (::EC_NV_AUTO_FOCUS_ACHIEVED)
     * - Auto focus timed out (::EC_NV_AUTO_FOCUS_TIMED_OUT)
     * - Auto exposure achieved (::EC_NV_AUTO_EXPOSURE_ACHIEVED)
     * - Auto exposure timed out (::EC_NV_AUTO_EXPOSURE_TIMED_OUT)
     * - Auto white balance achieved (
     *      ::EC_NV_AUTO_WHITE_BALANCE_ACHIEVED)
     * - Auto white balance timed out (
     *      ::EC_NV_AUTO_WHITE_BALANCE_TIMED_OUT)
     * 
     * See documentation for these events for information on how the
     * events can be trapped by applications.
     * 
     * Although half press is used with auto focus, auto exposure, and
     * auto white balance properties, an application can choose to
     * enable any one or any combination of these properties. When
     * any of these properties is disabled then half press will not
     * attempt to achieve that property.
     * 
     * Setting auto focus, auto exposure, and auto white balance:
     * 
     * - To enable auto focus, set NvCameraProperties::AutoFocus 
     *      to NV_TRUE, and set it using
     *      ::NvPropertyId_CameraProperties property ID.
     * - To enable auto exposure, set NvCameraProperties::ExposureTime 
     *      to ::NV_EXPOSURETIME_AUTO, and set it using
     *      \c NvPropertyId_CameraProperties property ID.
     * - To enable auto white balance, set NvCameraProperties::WhiteBalance 
     *      to ::NvWhiteBalance_Auto, and set it using
     *      \c NvPropertyId_CameraProperties property ID.
     * 
     * @par See also
     * Custom Events
     */
    NvPropertyId_HalfPress,

    /**
     * Filter-level property to set timeout value for half press.
     * Property value type is NvU32.
     * 
     * Requires a MAX time-out value in miliseconds, for which the camera can
     * converge (achieve) auto focus, auto exposure, and/or auto white balance
     * property before timing out.
     * 
     * This is valid only when ::NvPropertyId_HalfPress is NV_TRUE, and should
     * be set before \c NvPropertyId_HalfPress.
     */
    NvPropertyId_HalfPressTimeOut,

    /**
     * Filter-level property to enable Exif information.
     * Property value type is NvBool. Set to NV_TRUE for NVIDIA Video
     * Capture Filter to generate Exif information. This information
     * is used by NVIDIA Video Encoder Filter to embed Exif
     * information in JPEG images.
     */
    NvPropertyId_EnableExifInfo,

    /**
     * Filter-level property to enable pre capture converge.
     * Property value type is NvBool. This property functions
     * similar to half press. When set to NV_TRUE, NVIDIA Video
     * Capture Filter will achieve auto focus, auto exposure, or
     * auto white balance even before delivering the first frame.
     *
     * @pre This property should be set before connection.
     *
     * @sa     NvPropertyId_HalfPress
     */
    NvPropertyId_EnablePreCaptureConverge,

    /**
     * Filter-level property to change recording volume.
     * Property value type is NvU32 and is for NVIDIA Audio Capture
     * Filter. Property values range between 0 to UINT max.
     */
    NvPropertyId_RecordVolType,

    /**
     * Filter-level property to set automatic gain control (AGC).
     * Property value type is ::NvRecordAgcProperties. This property is
     * for NVIDIA Audio Capture Filter.
     */
    NvPropertyId_RecordAGC,

    /**
     * Filter-level property to set parametric equalizer filter.
     * Property value type is ::NvParametricEqFilterProperties. This
     * property is for NVIDIA Audio Capture Filter.
     */
    NvPropertyId_ParametricEqFilter,

    /**
     * Filter-level property to set pause after still capture feature.
     * Property value type is NvBool. When set to NV_TRUE, NVIDIA
     * Video Capture Filter sends an
     * ::EC_NV_AUTO_PAUSE_STILL_CAPTURE_DONE event to the application
     * whenever a still capture is triggered. This event can be
     * used by the application to pause the filter graph, ask the user
     * for save/delete confirmation, and run the filter graph again.
     */
    NvPropertyId_PausePreviewAfterStillCapture,

    /**
     * Filter-level property to enable preview stream.
     * Property value type is NvBool.
     */
    NvPropertyId_CameraAttributePreview,

    /**
     * Filter-level property to set white balance correlated color
     * temperature (CCT) range. Property value type is ::NvCaptureRange.
     * Auto white balance will normally search a large set of illuminants to
     * estimate the CCT of a scene. This range can be limited to a subset
     * of the defined illuminants by setting this parameter. The
     * \c NvCaptureRange structure's low and high values in the range
     * specify the minimum and maximum CCT values to be searched. Legal values
     * are integers in units of degrees K. To select a specific illuminant,
     * the low and high members should be set to identical values. By default,
     * the low range value is 0, and the high value is MAX_INT.
     */
    NvPropertyId_WhiteBalanceCCTRange,

    /**
     * Filter-level property to set file base number. This is required
     * for burst mode image capture. The base number will be incremented
     * and appended to the base file name for each burst mode image
     * captured. This is set on NVIDIA Mux File Writer.
     * Property value type is NvU32.
     * Property values range between 0 to UINT max.
     */
    NvPropertyId_FileBaseNumber,

    /**
     * Filter-level property to set file naming mode.
     * We use DCM File naming standard. To disable this 
     * mode, set this property on NVIDIA Mux File Writer.
     * Property value type is NvBool.
     * Property values is NV_FALSE to disable DCM file naming.
     */
    NvPropertyId_DCFFileMode,

    /**
     * Mux filter-level property to set the upper limit for encoded file size.
     * Property value type is NvU64. Size must be specified in bytes.
     * Default upper limit of encoded file size is 2 GB.
     */
    NvPropertyId_EncodedFileSize,

    /**
     * Filter-level property to set number of JPEG images to be captured in
     * burst mode. Set this property on NVIDIA Video Capture Filter.
     * Property value type is NvU32.
     * To cancel an on going burst capture, set count to 0 on this property and
     * trigger still pin.
     */
    NvPropertyId_JpegBurstModeImageCount,
    
    /**
     * Capture filter property to describe camera metering mode.
     * Metering modes are valid only when auto exposure (AE) is specified,
     * i.e., not valid for manual exposure. When AE is not specified,
     * spot metering mode is the default. Metering mode values
     * are defined in the ::NvCameraMeteringMode enumeration.
     */
    NvPropertyId_CameraMeteringMode,

    /** 
     * Filter-level property to get the number of audio frames written.
     * This is a read-only property supported by NVIDIA Mux File Writer. 
     * Property value type is NvU64. 
     */
    NvPropertyId_AudioEncodedFramesWritten,

    /** 
     * Filter-level property to get the number of video frames written .
     * This is a read-only property supported by NVIDIA Mux File Writer. 
     * Property value type is NvU64. 
     */
    NvPropertyId_VideoEncodedFramesWritten,
    
    /**
     * Filter-level property to get/set the camera profile. Applications can 
     * enable profile before Dshow graph runs and get profile data at runtime, 
     * but this property does not support setting it after capture is runing.
     * Property value type is ::NvCaptureProfileRec.
     */
    NvPropertyId_CameraProfile,

    /**
     * NVIDIA Mux File Writer property to reinitialize filename upon receipt
     * of ::EC_NV_MUX_WRITER_MORE_DATA_REJECT or 
     * ::EC_NV_MUX_WRITER_FILESIZE_LIMIT_EXCEED events.
     *
     * This property can also be used by the application to have a new file 
     * created after every camera stream pause, followed by setting the property 
     * to NVIDIA Video Encoder to \c ENCODER_VIDEO_IFRAME_INSERT. So in this case,
     * the programming sequence would look like the following:
     * After resumption from \c ControlStream(Stop), i.e. on 
     * \c ControlStream(Start):
     * -# setProperty ::NvPropertyId_MuxWriterReinitializeFilename
     * -# setProperty ::ENCODER_VIDEO_IFRAME_INSERT to encoder 
     *
     * This is a write-only property supported by NVIDIA Mux File Writer.
     */
    NvPropertyId_MuxWriterReinitializeFilename,

    /**
     * Filter-level property to get/set frame rate range of NVIDIA Video
     * capture filter. Property value type is ::NvCaptureRange *. Sets the
     * minimum and maximum frame rate at which the capture filter delivers
     * frames.
     *
     * NVIDIA Video Capture filter supports auto frame rate, where it
     * dynamically changes frame rate between a minimum value specified by
     * \c NvCaptureRange::Low and a maximum value specified by
     * \c NvCaptureRange::High. To make the capture filter deliver frames at
     * a constant frame rate, set \c Low and \c High members of \c NvCaptureRange
     * structure to the same value.
     *
     * Maximum possible frame rate is dependent on the camera sensor. Values
     * above the maximum frame rate supported by the camera sensor are clipped
     * to the maximum value.
     */
    NvPropertyId_FrameRateRange,

     /**
     * Filter-level property to get a block of custom info from the imager.
     * Property value type is ::NvCaptureCustomInfo. This is a read-only property.
     * The block of info that is returned is private and is known to the NvODM
     * implementation.
     */
    NvPropertyId_CustomizedBlockInfo,

    /**
     * Filter-level property to get/set exposure time range of NVIDIA Video
     * capture filter. Property value type is NvCaptureRange *. Sets the
     * minimum and maximum exposure time or iso sensitivty at which the
     * capture filter's auto exposure picks values from.
     *
     * NVIDIA Video Capture filter supports auto exposure, where it
     * dynamically changes exposure time and sensor's sensitivty between a
     * minimum value specified by NvCaptureRange::Low and a
     * maximum value specified by \c NvCaptureRange::High. 
     *
     * Minimum and maximum possible exposure times and iso gains are dependent
     * on the camera sensor. Values above the maximum and below the minimum 
     * supported by the camera sensor are clipped.
     * 
     * For ExposureTimeRange:
     * Values are in units of milliseconds. To limit the exposure to nothing
     * slower than 1/30 second, a setting like {0, 33} would be used. Zero would
     * be replaced with the sensors fastest exposure limit by the driver.
     * For ISORange:
     * Values are in units of ISO. To limit the ISO gain to nothing more than
     * ISO400, a setting like {0, 400} would be used. Zero would be replaced
     * with the sensor's lowest supported gain limit by the driver.
     * Since the Auto exposure algorithm is 'exposure time priority', limiting
     * the ISO range will only dim the image if the ISO is limited by setting
     * the range. The exposure time is not recalculated based off of the
     * ISO range limit.
     */
    NvPropertyId_ExposureTimeRange,
    NvPropertyId_ISORange,

    /**
     * Filter-level property to get/set how much flash white balance
     * correction to use when capturing flashed images. The property type
     * is NvF32, and the range is 0 to 1. For example, at .30,
     * it would use 30% of the flash color correction and 70% of the
     * previously-calculated ambient color correction.
     *
     * @note If the ambient scene was very dark, AWB may choose to
     * mix in more flash correction than specified. Setting this property
     * to zero disables this behavior.
     */
    NvPropertyId_FlashMixingRatio,

    /**
     * NVIDIA Mux File Writer property to program 32 bit Atom compatible 
     * 3GP Files. Default is 64 bit Atom. To Enable 32 bit atom Mode
     * This property needs to be set to TRUE, i.e boolean value one.
     * This is a write-only property supported by NVIDIA Mux File Writer.
     */
    NvPropertyId_MuxWriterEnable64To32BitAtomMode,

    /**
     * Filter-level property to set warmth.
     * Property value type is NvF32. NVIDIA Video Capture Filter
     * supports warmth values from -10 to 10
     */
    NvPropertyId_Warmth,

     /**
     * NVIDIA Mux File Writer property to set user data to go in Udta 
     * Atom of 3GP/MP4 sink files.
     * Application can define any of the userconfig type as stated in enum
     * Nv3GPWriterUserDataType to go in Udta Atom of 3GP/MP4.
     * Apllication will setproperty to MUX_WRITER with BYREF Variant , passing 
     * on pointer of completely/properly filled in structure
     * NvWriterUserDataConfig. 
     * It is sole responsiblity of the application to supply right
     * data in NvWriterUserDataConfig::pUserData as it will simply be sticthed 
     * in the sink 3GP/Mp4 file by NVIDIA FileWriter without 
     * checking/processing its content, there upon if the resulting 3GP File is 
     * not playable that will imply something is wrong in application supplied
     * NvWriterUserDataConfig::pUserData.
     * 
     */
    NvPropertyId_MuxWriterUserData,

    /**
     * While the auto exposure algorithm is running it will internally track
     * the calculated brightness of the scene, which is the product of the
     * average scene luminance and the reciprocal of the exposure settings
     * used. If a scene is low key but in bright light, this value will be
     * smaller than a high key scene in bright light. The lowest value will
     * be low key scenes with very long exposures; the highest value will be
     * high key scenes in bright light. They property type is NvF32 and this
     * is read-only.
     */
     NvPropertyId_SceneBrightness,

    /**
     * Setting the thumbnail quality improvement. When it's set to off, there
     * won't be any improvement. When it's set to best/fast, thumbnail quality
     * improvement will be applied to the still capture and every frame of 
     * a burst capture. When it's set to LastFrameBest/LastFrameFastest,
     * thumbnail quality improvement will be applied to the still capture and
     * the last frame of a burst capture.
     * The property type is NvThumbnailQualityImprovement.
     */
    NvPropertyId_ThumbnailQualityImprovement,

    /**
     * Filter-level property to set the convergence speed of the AE and AWB
     * algorithms. It is a write-only property.
     *
     * Property type: NvU8*
     *
     * Only the specified two strings are allowed :
     * - "ae.ConvergeSpeed=%f"
     * - "awb.ConvergeSpeed=%f"
     *
     * where %f is a float value.
     *
     * The float value for 
     * - ae must be [0.01-1.0] 
     * - awb must be [0.01-1.0]
     *
     * If the strings passed do not match any of the above 2 strings then
     * an error is returned.
     */
    NvPropertyId_CameraConfiguration,

    /** Ignore -- Forces compilers to make 32-bit enums. */
    NvPropertyId_Force32 = 0x7FFFFFFF
} NvPropertyId;


/**
 * @brief       1 second in 100s of nano-seconds
 */
#define UNITS 10000000

/**
 * @brief       Bit counts of YUV420 and RGB565
 */
enum
{
    VIDEO_BIT_COUNT_YUV420 = 12,
    VIDEO_BIT_COUNT_RGB565 = 16
};

/**
 * @brief       Enumeration for pin index of camera source
 */
enum
{
    PIN_INDEX_PREVIEW,
    PIN_INDEX_CAPTURE
};


/**
 * @brief       Enumeration for value to be added to the
 *              duration for which the graph waits before
 *              stopping.
 */
enum { ADDITIONAL_TIME = 15 };

/**
 * @brief       The list of strings that contain a list of file formats
 */
WCHAR NvFileFormatString[][4] =
{
    L"3gp",
    L"mp4",
    L"264",
    L"m4v",
    L"263",
    L"amr",
    L"awb",
    L"aac",
    L"jpg"
};

/**
 * @brief       Enumeration for types of file formats
 */
typedef enum
{
    NvFileFormat_3gp = 1,
    NvFileFormat_Mp4,
    NvFileFormat_H264,
    NvFileFormat_M4v,
    NvFileFormat_H263,
    NvFileFormat_Amr,
    NvFileFormat_Awb,
    NvFileFormat_Aac,
    NvFileFormat_Jpeg,

    NvFileFormat_Force32 = 0x7FFFFFFF
} NvFileFormat;

enum
{
    STREAM_YES = 1000,
    STREAM_NO = 1001
};

#endif  // INCLUDED_ENCODE_TEST_DEFINES_H
