

#include "sensor_common.h"
#include "nvodm_imager.h"
//#include "nvodm_imager_int.h"
#include "imager_hal.h" //shard ressources with driver
#include <windows.h>
#include "I2CLib.h"
#include "CoProcLib.h"

#define	DEBUG_ERR				0x1
#define DEBUG_WARN				0x2
#define DEBUG_FUNC				0x4
#define DEBUG_INIT				0x8
#define DEBUG_INFO   			0x10
#define DEBUG_VERBOSE			0x20



DBGPARAM dpCurSettings =
{
    TEXT("CameraPlugin"),
	{
		TEXT("Errors"),    TEXT("Warnings"),  TEXT("Function"),	   TEXT("Init"),
		TEXT("Interrupt"), TEXT("Info"),      TEXT("Verbose"),   TEXT("Undefined"),
		TEXT("Undefined"), TEXT("Undefined"), TEXT("Undefined"),   TEXT("Undefined"),
		TEXT("Undefined"), TEXT("Undefined"), TEXT("Undefined"),   TEXT("Undefined")
    },
	DEBUG_ERR|DEBUG_WARN|DEBUG_FUNC|DEBUG_INIT|DEBUG_INFO
};




BYTE g_SensorI2C_Addr;
DWORD g_SensorI2C_OffsetWidth = 1; // 1 or 2 bytes
DWORD g_SensorI2C_DataWidth = 1; // 1 or 2 bytes
DWORD g_SensorI2C_Speed = I2C_100KBPS;

NvBool SensorI2C_Open()
{
	DWORD VerMaj;
	DWORD VerMin;
	DWORD VerBuild;
    BOOL ret;

	I2cIntGetLibVersion(&VerMaj, &VerMin, &VerBuild);
	DEBUGMSG(ZONE_FUNC && ZONE_VERBOSE, (_T("+SensorI2C_Open, I2C Library Version %d.%d Build %d\n"), VerMaj, VerMin, VerBuild));

	if(getModuleFamily() == 1) // Apalis
		ret = I2CInitEx(0x7000C500);
	else // Colibri
		ret = I2CInit();

    if(!ret) goto fail;

	ret = GetI2CLock(500);
    if(!ret) goto fail;

	I2CSetSpeed(g_SensorI2C_Speed);

    DEBUGMSG(ZONE_FUNC && ZONE_VERBOSE, (_T("-SensorI2C_Open: OK\n")));
    return TRUE;
fail:
    DEBUGMSG(ZONE_ERR, (_T("-SensorI2C_Open: FAILED\n")));
    SensorI2C_Close();
    return FALSE;
}

void SensorI2C_Close()
{
    DEBUGMSG(ZONE_FUNC && ZONE_VERBOSE, (_T("+SensorI2C_Close\n")));
    ReleaseI2CLock();
    I2CDeInit();
    DEBUGMSG(ZONE_FUNC && ZONE_VERBOSE, (_T("-SensorI2C_Close\n")));
}

NvBool SensorI2C_Write(WORD offset, WORD data)
{
    BOOL ret;

    if(g_SensorI2C_DataWidth==2) // Swap as we need to send MSB first
        data=(data<<8)|(data>>8);

    if(g_SensorI2C_OffsetWidth==1)
    {
        ret = I2CBurstWrite(g_SensorI2C_Addr, (BYTE*)&data, offset, g_SensorI2C_DataWidth);
    }
    else // 16Bit Offset
    {
        WORD payload[2];
		payload[0] = (offset << 8)|(offset >> 8);	// Swap as we need to send MSB first
		payload[1] = data;
        ret = I2CBurstWrite(g_SensorI2C_Addr, (BYTE*)&payload, -1, g_SensorI2C_OffsetWidth+g_SensorI2C_DataWidth);
    }

    return ret;
}

NvBool SensorI2C_Read(WORD offset, WORD* pData)
{
    BOOL ret;

    *pData=0;

    if(g_SensorI2C_OffsetWidth==1)
        ret = I2CBurstRead(g_SensorI2C_Addr, (BYTE*)pData, offset, g_SensorI2C_DataWidth);
    else
    {
        offset = (offset << 8)|(offset >> 8);	// Swap as we need to send MSB first
        // I2C library does not support reading with 16bit offsets, so we need to do a Write and a Read (might not work on all sensors)
        ret = I2CBurstWrite(g_SensorI2C_Addr, (BYTE*)&offset, -1, 2);
        ret &= I2CBurstRead(g_SensorI2C_Addr, (BYTE*)pData, -1, g_SensorI2C_DataWidth);
    }

    if(g_SensorI2C_DataWidth==2) // Swap as we need to send MSB first
        *pData=(*pData<<8)|(*pData>>8);

    return ret;
}

NvBool SensorI2C_WriteSequence(DevCtrlReg16* SetModeSequence)
{
    DevCtrlReg16 *ptr = SetModeSequence;
    NvBool ret= TRUE;

    DEBUGMSG(ZONE_FUNC, (L"+SensorI2C_WriteSequence\n"));

    ret = SensorI2C_Open();
    if (ret)
    {
        while(ptr->RegAddr!= SEQUENCE_END)
        {
            if(ptr->RegAddr == SEQUENCE_WAIT_MS){
                Sleep(ptr->RegValue);
            } else if(ptr->RegAddr == SEQUENCE_CHANGE_ADDR){
                g_SensorI2C_Addr = (BYTE)ptr->RegValue;
            } else{
                ret = SensorI2C_Write(ptr->RegAddr, ptr->RegValue);
                if(!ret)
                    break;
            }
            ptr += 1;
        }
        SensorI2C_Close();
    }

    DEBUGMSG(ZONE_FUNC, (L"-SensorI2C_WriteSequence\n"));
    return ret;
}

char* GetParamName(NvOdmImagerParameter Param)
{
    static char tmpStr[16];

    switch(Param)
    {
    case NvOdmImagerParameter_SensorExposure:
        return "NvOdmImagerParameter_SensorExposure";
    case NvOdmImagerParameter_SensorGain:
        return "NvOdmImagerParameter_SensorGain";
    case NvOdmImagerParameter_SensorFrameRate:
        return "NvOdmImagerParameter_SensorFrameRate";
    case NvOdmImagerParameter_MaxSensorFrameRate:
        return "NvOdmImagerParameter_MaxSensorFrameRate";
    case NvOdmImagerParameter_SensorInputClock:
        return "NvOdmImagerParameter_SensorInputClock";
    case NvOdmImagerParameter_FlashCapabilities:
        return "NvOdmImagerParameter_FlashCapabilities";
    case NvOdmImagerParameter_FocalLength:
        return "NvOdmImagerParameter_FocalLength";
    case NvOdmImagerParameter_MaxAperture:
        return "NvOdmImagerParameter_MaxAperture";
    case NvOdmImagerParameter_FNumber:
        return "NvOdmImagerParameter_FNumber";
    case NvOdmImagerParameter_SensorExposureLimits:
        return "NvOdmImagerParameter_SensorExposureLimits";
    case NvOdmImagerParameter_SensorGainLimits:
        return "NvOdmImagerParameter_SensorGainLimits";
    case NvOdmImagerParameter_SensorFrameRateLimits:
        return "NvOdmImagerParameter_SensorFrameRateLimits";
    case NvOdmImagerParameter_SensorExposureLatchTime:
        return "NvOdmImagerParameter_SensorExposureLatchTime";
    case NvOdmImagerParameter_CalibrationData:
        return "NvOdmImagerParameter_CalibrationData";
    case NvOdmImagerParameter_CalibrationOverrides:
        return "NvOdmImagerParameter_CalibrationOverrides";
    case NvOdmImagerParameter_SelfTest:
        return "NvOdmImagerParameter_SelfTest";
    case NvOdmImagerParameter_DeviceStatus:
        return "NvOdmImagerParameter_DeviceStatus";
    case NvOdmImagerParameter_OptimizeResolutionChange:
        return "NvOdmImagerParameter_OptimizeResolutionChange";
    case NvOdmImagerParameter_StereoCapable:
        return "NvOdmImagerParameter_StereoCapable";
    case NvOdmImagerParameter_SensorInherentGainAtResolution:
        return "NvOdmImagerParameter_SensorInherentGainAtResolution";
    case NvOdmImagerParameter_ISPSetting:
        return "NvOdmImagerParameter_ISPSetting";
    case NvOdmImagerParameter_OperationalMode:
        return "NvOdmImagerParameter_OperationalMode";
    case NvOdmImagerParameter_FactoryCalibrationData:
        return "NvOdmImagerParameter_FactoryCalibrationData";
    case NvOdmImagerParameter_SensorGroupHold:
        return "NvOdmImagerParameter_SensorGroupHold";
    case NvOdmImagerParameter_GetBestSensorMode:
        return "NvOdmImagerParameter_GetBestSensorMode";

    case NvOdmImagerParameter_SensorTriggerStill:
        return "NvOdmImagerParameter_SensorTriggerStill";

    default:
        sprintf_s(tmpStr, 16, "%d", Param);
        return tmpStr;
    }
}

static DWORD s_SensorIndex = 3; // Default to first Sensor in the List

typedef struct SensorHalRec
{
    char* pName;
    pfnImagerGetHal pfnGetHal;
} SENSOR_HAL;

NvBool SensorPlugin_GetHal_Bayer_MT9P031(NvOdmImagerHandle hImager);
NvBool SensorPlugin_GetHal_Bayer_OV2640(NvOdmImagerHandle hImager);
NvBool SensorPlugin_GetHal_YUV_ADV7180(NvOdmImagerHandle hImager);
NvBool SensorPlugin_GetHal_YUV_ADV7280(NvOdmImagerHandle hImager);
NvBool SensorPlugin_GetHal_YUV_MAX9526(NvOdmImagerHandle hImager);
NvBool SensorPlugin_GetHal_YUV_MT9V111(NvOdmImagerHandle hImager);
NvBool SensorPlugin_GetHal_YUV_OV5642(NvOdmImagerHandle hImager);
NvBool SensorPlugin_GetHal_YUV_OV5640(NvOdmImagerHandle hImager);
NvBool SensorPlugin_GetHal_YUV_OV5640_Parallel(NvOdmImagerHandle hImager);

static SENSOR_HAL s_HalList[] = {
    {"Bayer MT9P031", SensorPlugin_GetHal_Bayer_MT9P031},
    {"Bayer OV2640", SensorPlugin_GetHal_Bayer_OV2640},
    {"YUV ADV7180 (ACM)", SensorPlugin_GetHal_YUV_ADV7180},
    {"YUV ADV7280 (ACA)", SensorPlugin_GetHal_YUV_ADV7280},
    {"YUV MAX9526", SensorPlugin_GetHal_YUV_MAX9526},
    {"YUV MT9V111", SensorPlugin_GetHal_YUV_MT9V111},
    {"YUV OV5642", SensorPlugin_GetHal_YUV_OV5642},
    {"YUV OV5640 CSI", SensorPlugin_GetHal_YUV_OV5640},
	{"YUV OV5640 Parallel", SensorPlugin_GetHal_YUV_OV5640_Parallel},
    {NULL, NULL},
};
#define SENSOR_COUNT (sizeof(s_HalList)/sizeof(s_HalList[0])-1)

__declspec(dllexport) SENSOR_HAL* SensorPlugin_GetList()
{
    return s_HalList;
}

/*__declspec(dllexport) NvBool SensorPlugin_SetSensorIndex(DWORD sensorIndex)
{
    if(sensorIndex >= SENSOR_COUNT)
        return FALSE;

    s_SensorIndex = sensorIndex;
    return TRUE;
}*/

void SelectSensor(void)
{
    HKEY hKey = NULL;
    DWORD dwDisp;
    TCHAR szSensorName[64];
    DWORD dwDataSize = sizeof(szSensorName);

    szSensorName[0]=0;

    // Check for SensorName in Registry
    if (ERROR_SUCCESS != RegCreateKeyEx(HKEY_LOCAL_MACHINE, L"SOFTWARE\\NVIDIA Corporation\\NVCAM", 0, NULL, 0, 0, NULL, &hKey, &dwDisp))
        return;
    RegQueryValueEx(hKey, L"SensorName", NULL, NULL, (LPBYTE)&szSensorName, &dwDataSize);

    // Search for a match in s_HalList
    {
        TCHAR szListEntry[256];
        SENSOR_HAL* pSensorList = s_HalList;
        DWORD sensorIndex=0;
        // Build a list of all Sensors. if we find one named SensorName we abort and exit
        while(s_HalList[sensorIndex].pName)
        {
            swprintf_s(szListEntry, sizeof(szListEntry)/2, L"%S", s_HalList[sensorIndex].pName);
            if(szSensorName[0] && wcsstr(szListEntry, szSensorName)) {s_SensorIndex = sensorIndex; break;}
            sensorIndex++;
        }
    }
}

DWORD GetSensorMode()
{
    HKEY hKey = NULL;
    DWORD dwDisp;
    DWORD SensorMode = -1; // -1 Means autoselect

    if (ERROR_SUCCESS == RegCreateKeyEx(HKEY_LOCAL_MACHINE, L"SOFTWARE\\NVIDIA Corporation\\NVCAM", 0, NULL, 0, 0, NULL, &hKey, &dwDisp))
    {
        DWORD dwDataSize = sizeof(DWORD);
        RegQueryValueEx(hKey, L"SensorMode", NULL, NULL, (LPBYTE)&SensorMode, &dwDataSize);
        RegCloseKey(hKey);
    }

    return SensorMode;
}

__declspec(dllexport) NvBool SensorPlugin_GetHal(NvOdmImagerHandle hImager)
{

    SelectSensor();

    if(s_SensorIndex >= SENSOR_COUNT)
        return FALSE;

    return s_HalList[s_SensorIndex].pfnGetHal(hImager);
}
BOOL APIENTRY DllMain(HANDLE hModule,
    DWORD  ul_reason_for_call,
    LPVOID lpReserved
)
{
    return TRUE;
}
